package PuellaMagi.mods;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityCreature;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.monster.EntityGiantZombie;
import net.minecraft.entity.monster.EntityMob;
import net.minecraft.entity.passive.IAnimals;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.DamageSource;
import net.minecraft.util.MathHelper;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.util.Vec3;
import net.minecraft.world.ChunkPosition;
import net.minecraft.world.EnumSkyBlock;
import net.minecraft.world.World;

public class EntityGriefSeed extends EntityCreature implements IAnimals
{
    private int xTile = -1;
    private int yTile = -1;
    private int zTile = -1;
    private int inTile = 0;
    private int inData = 0;
    private boolean inGround = false;
    public Entity shootingEntity;
    public int arrowShake = 0;
    private int ticksInGround;
    private int ticksInAir = 0;
    private List teritoryBlockPositions = new ArrayList();

    public EntityGriefSeed(World par1World)
    {
        super(par1World);
        this.setSize(0.5F, 0.5F);
        if (this.getCountDown() == -1)
        {
        	this.setThrowableHeading(this.rand.nextDouble() - 0.5F, this.rand.nextDouble(), this.rand.nextDouble() - 0.5F, 1.6F, 0.0F);
        }
    }

    public EntityGriefSeed(World par1World, EntityLiving par2EntityLiving, float par3)
    {
        this(par1World);
        this.shootingEntity = par2EntityLiving;
        this.setLocationAndAngles(par2EntityLiving.posX, par2EntityLiving.posY + (double)par2EntityLiving.getEyeHeight(), par2EntityLiving.posZ, par2EntityLiving.rotationYaw, par2EntityLiving.rotationPitch);
        this.posX -= (double)(MathHelper.cos(this.rotationYaw / 180.0F * (float)Math.PI) * 0.26F);
        this.posY -= 0.10000000149011612D;
        this.posZ -= (double)(MathHelper.sin(this.rotationYaw / 180.0F * (float)Math.PI) * 0.26F);
        this.setPosition(this.posX, this.posY, this.posZ);
        this.yOffset = 0.0F;
        this.motionX = (double)(-MathHelper.sin(this.rotationYaw / 180.0F * (float)Math.PI) * MathHelper.cos(this.rotationPitch / 180.0F * (float)Math.PI));
        this.motionZ = (double)(MathHelper.cos(this.rotationYaw / 180.0F * (float)Math.PI) * MathHelper.cos(this.rotationPitch / 180.0F * (float)Math.PI));
        this.motionY = (double)(-MathHelper.sin(this.rotationPitch / 180.0F * (float)Math.PI));
        this.setThrowableHeading(this.motionX, this.motionY, this.motionZ, par3 * 1.5F, 1.0F);
    }

    @Override
    public int getMaxHealth()
    {
        return 10;
    }

    public void setThrowableHeading(double par1, double par3, double par5, float par7, float par8)
    {
        float var9 = MathHelper.sqrt_double(par1 * par1 + par3 * par3 + par5 * par5);
        par1 /= (double)var9;
        par3 /= (double)var9;
        par5 /= (double)var9;
        par1 += this.rand.nextGaussian() * 0.007499999832361937D * (double)par8;
        par3 += this.rand.nextGaussian() * 0.007499999832361937D * (double)par8;
        par5 += this.rand.nextGaussian() * 0.007499999832361937D * (double)par8;
        par1 *= (double)par7;
        par3 *= (double)par7;
        par5 *= (double)par7;
        this.motionX = par1;
        this.motionY = par3;
        this.motionZ = par5;
        float var10 = MathHelper.sqrt_double(par1 * par1 + par5 * par5);
        this.prevRotationYaw = this.rotationYaw = (float)(Math.atan2(par1, par5) * 180.0D / Math.PI);
        this.prevRotationPitch = this.rotationPitch = (float)(Math.atan2(par3, (double)var10) * 180.0D / Math.PI);
        this.ticksInGround = 0;
    }

    @Override
    public void setPositionAndRotation2(double par1, double par3, double par5, float par7, float par8, int par9)
    {
        this.setPosition(par1, par3, par5);
        this.setRotation(par7, par8);
    }

    @Override
    public void setVelocity(double par1, double par3, double par5)
    {
        this.motionX = par1;
        this.motionY = par3;
        this.motionZ = par5;

        if (this.prevRotationPitch == 0.0F && this.prevRotationYaw == 0.0F)
        {
        	float var7 = MathHelper.sqrt_double(par1 * par1 + par5 * par5);
        	this.prevRotationYaw = this.rotationYaw = (float)(Math.atan2(par1, par5) * 180.0D / Math.PI);
        	this.prevRotationPitch = this.rotationPitch = (float)(Math.atan2(par3, (double)var7) * 180.0D / Math.PI);
        	this.prevRotationPitch = this.rotationPitch;
        	this.prevRotationYaw = this.rotationYaw;
        	this.setLocationAndAngles(this.posX, this.posY, this.posZ, this.rotationYaw, this.rotationPitch);
        	this.ticksInGround = 0;
        }
    }

    @Override
    protected String getDeathSound()
    {
        return "random.break";
    }

    @Override
    public void onLivingUpdate()
    {
        if (this.worldObj.getClosestPlayer(this.posX + 0.5D, this.posY + 0.5D, this.posZ + 0.5D, 16.0D) != null)
        {
        	if (this.getCountDown() == -1)
        	{
        		this.setNewCountDown();
        	}
        	if (this.getCountDown() > 0)
        	{
        		this.startCountDown();
        	}
        	else
        	{
        		this.spawnMajoFromGrief();
        	}
        	if (this.worldObj.isRemote)
        	{
        		if (this.getCountDown() < 100)
        		{
        			double var1 = (double)((float)this.posX + this.rand.nextFloat() - 0.5F);
        			double var3 = (double)((float)this.posY + this.rand.nextFloat());
        			double var5 = (double)((float)this.posZ + this.rand.nextFloat() - 0.5F);
        			this.worldObj.spawnParticle("reddust", var1, var3, var5, this.rand.nextDouble(), this.rand.nextDouble(), this.rand.nextDouble());
        		}
        		if (this.getCountDown() < 500)
        		{
        			double var1 = (double)((float)this.posX + this.rand.nextFloat() - 0.5F);
        			double var3 = (double)((float)this.posY + this.rand.nextFloat());
        			double var5 = (double)((float)this.posZ + this.rand.nextFloat() - 0.5F);
        			this.worldObj.spawnParticle("reddust", var1, var3, var5, 0.001D, 0.001D, 0.001D);
        		}
        		if (this.getCountDown() < 1000 && this.getCountDown() % 5 == 0)
        		{
        			double var1 = (double)((float)this.posX + this.rand.nextFloat() - 0.5F);
        			double var3 = (double)((float)this.posY + this.rand.nextFloat());
        			double var5 = (double)((float)this.posZ + this.rand.nextFloat() - 0.5F);
        			this.worldObj.spawnParticle("reddust", var1, var3, var5, 0.001D, 0.001D, 0.001D);
        		}
        	}
        }
    }

    public void spawnMajoFromGrief()
    {
        EntityMob var0 = this.chooseMajo(this.worldObj);
        if (var0 != null)
        {
        	if (!this.randomSpawnMajo(var0))
        	{
        		this.forcedSpawnMajo(var0);
        	}
        }
    }

    public boolean randomSpawnMajo(EntityMob par1EntityMob)
    {
        for (int var10 = 0; var10 < 16; var10++)
        {
        	double var4 = this.posX + (this.worldObj.rand.nextDouble() - this.worldObj.rand.nextDouble()) * 4.0D;
        	double var6 = this.posY + (double)(this.rand.nextInt(3) - 1);
        	double var8 = this.posZ + (this.worldObj.rand.nextDouble() - this.worldObj.rand.nextDouble()) * 4.0D;
        	par1EntityMob.setLocationAndAngles(var4, var6, var8, this.worldObj.rand.nextFloat() * 360.0F, 0.0F);
        	if (par1EntityMob.getCanSpawnHere())
        	{
        		if (!this.worldObj.isRemote)
        		{
        			this.worldObj.spawnEntityInWorld(par1EntityMob);
        		}
        		par1EntityMob.spawnExplosionParticle();
        		this.setDead();
        		return true;
        	}
        }
        return false;
    }

    public void forcedSpawnMajo(EntityMob par1EntityMob)
    {
        par1EntityMob.setPositionAndRotation(this.posX, this.posY, this.posZ, this.rotationPitch, 0.0F);
        int var1 = 0;
        int var10 = (int)par1EntityMob.height;
        HashSet var11 = new HashSet();
        while (true)
        {
        	if (par1EntityMob.getCanSpawnHere() || var1 * 2 > var10)
        	{
        		if (!this.worldObj.isRemote)
        		{
        			this.worldObj.spawnEntityInWorld(par1EntityMob);
        		}
        		par1EntityMob.spawnExplosionParticle();
        		this.setDead();
        		return;
        	}
        	for (int var2 = -var1; var2 <= var1; var2++)
        	{
        		for (int var3 = 0; var3 <= var1 * 2; var3++)
        		{
        			for (int var4 = -var1; var4 <= var1; var4++)
        			{
        				if (var2 == -var1 || var2 == var1 || var3 == var1 * 2 -1 || var3 == var1 * 2 || var4 == -var1 || var4 == var1)
        				{
        					int var5 = (int)this.posX + var2;
        					int var6 = (int)this.posY + var3;
        					int var7 = (int)this.posZ + var4;
        					var11.add(new ChunkPosition(var5, var6, var7));
        				}
        			}
        		}
        	}
        	this.teritoryBlockPositions.addAll(var11);
        	this.breakTeritoryBlock();
        	var1++;
        }
    }

    private void breakTeritoryBlock()
    {
        Iterator var1 = this.teritoryBlockPositions.iterator();
        while (var1.hasNext())
        {
        	ChunkPosition var2 = (ChunkPosition)var1.next();
        	int var5 = var2.x;
        	int var6 = var2.y;
        	int var7 = var2.z;
        	int var8 = this.worldObj.getBlockId(var5, var6, var7);
        	if (var8 > 0 && var8 != Block.bedrock.blockID)
        	{
        		if (this.worldObj.setBlockWithNotify(var5, var6, var7, 0))
        		{
        			Block var9 = Block.blocksList[var8];
        			if (var8 == Block.dirt.blockID || var8 == Block.stone.blockID || var8 == Block.sand.blockID)
        			{
        				var9.dropBlockAsItemWithChance(this.worldObj, var5, var6, var7, this.worldObj.getBlockMetadata(var5, var6, var7), 0.001F, 0);
        			}
        			else
        			{
        				var9.dropBlockAsItemWithChance(this.worldObj, var5, var6, var7, this.worldObj.getBlockMetadata(var5, var6, var7), 0.1F, 0);
        			}
        			if (this.worldObj.setBlockAndMetadataWithUpdate(var5, var6, var7, 0, 0, this.worldObj.isRemote))
        			{
        				this.worldObj.notifyBlocksOfNeighborChange(var5, var6, var7, 0);
        			}
        		}
        	}
        }
        this.teritoryBlockPositions.clear();
    }

    public EntityMob chooseMajo(World par1World)
    {
        int var1 = par1World.rand.nextInt(1);
        switch(var1)
        {
        	case 0:
        		return new EntityGertrud(par1World);

        	default:
        		return new EntityGiantZombie(par1World);
        }
    }

    @Override
    protected void dropFewItems(boolean par1, int par2)
    {
        ItemStack var1 = new ItemStack(mod_QB.itemGriefSeed.shiftedIndex, 1, this.getSoulGemDamage());
        this.entityDropItem(var1, 0.0F);
    }

    @Override
    protected void entityInit()
    {
        super.entityInit();
        this.dataWatcher.addObject(20, new Integer(0));
        this.dataWatcher.addObject(21, new Integer(-1));
    }

    @Override
    public boolean attackEntityFrom(DamageSource par1DamageSource, int par2)
    {
        if (!this.worldObj.isRemote)
        {
        	Entity var1 = par1DamageSource.getEntity();
        	if (!(var1 instanceof EntityPlayer) && !(var1 instanceof EntityMahoShojo))
        	{
        		return false;
        	}
        }
        return super.attackEntityFrom(par1DamageSource, par2);
    }

    @Override
    public void onUpdate()
    {
        this.onEntityUpdate();
        this.onLivingUpdate();
        if (this.prevRotationPitch == 0.0F && this.prevRotationYaw == 0.0F)
        {
        	float var1 = MathHelper.sqrt_double(this.motionX * this.motionX + this.motionZ * this.motionZ);
        	this.prevRotationYaw = this.rotationYaw = (float)(Math.atan2(this.motionX, this.motionZ) * 180.0D / Math.PI);
        	this.prevRotationPitch = this.rotationPitch = (float)(Math.atan2(this.motionY, (double)var1) * 180.0D / Math.PI);
        }

        int var16 = this.worldObj.getBlockId(this.xTile, this.yTile, this.zTile);
        if (var16 > 0)
        {
        	Block.blocksList[var16].setBlockBoundsBasedOnState(this.worldObj, this.xTile, this.yTile, this.zTile);
        	AxisAlignedBB var2 = Block.blocksList[var16].getCollisionBoundingBoxFromPool(this.worldObj, this.xTile, this.yTile, this.zTile);

        	if (var2 != null && var2.isVecInside(this.worldObj.getWorldVec3Pool().getVecFromPool(this.posX, this.posY, this.posZ)))
        	{
        		this.inGround = true;
        	}
        }

        if (this.arrowShake > 0)
        {
        	--this.arrowShake;
        }

        if (this.inGround)
        {
        	int var18 = this.worldObj.getBlockId(this.xTile, this.yTile, this.zTile);
        	int var19 = this.worldObj.getBlockMetadata(this.xTile, this.yTile, this.zTile);

        	if (var18 == this.inTile && var19 == this.inData)
        	{
        		++this.ticksInGround;
        	}
        	else
        	{
        		this.inGround = false;
        		this.motionX *= (double)(this.rand.nextFloat() * 0.2F);
        		this.motionY *= (double)(this.rand.nextFloat() * 0.2F);
        		this.motionZ *= (double)(this.rand.nextFloat() * 0.2F);
        		this.ticksInGround = 0;
        		this.ticksInAir = 0;
        	}
        }
        else
        {
        	++this.ticksInAir;
        	Vec3 var17 = this.worldObj.getWorldVec3Pool().getVecFromPool(this.posX, this.posY, this.posZ);
        	Vec3 var3 = this.worldObj.getWorldVec3Pool().getVecFromPool(this.posX + this.motionX, this.posY + this.motionY, this.posZ + this.motionZ);
        	MovingObjectPosition var4 = this.worldObj.rayTraceBlocks_do_do(var17, var3, false, true);
        	var17 = this.worldObj.getWorldVec3Pool().getVecFromPool(this.posX, this.posY, this.posZ);
        	var3 = this.worldObj.getWorldVec3Pool().getVecFromPool(this.posX + this.motionX, this.posY + this.motionY, this.posZ + this.motionZ);
        	if (var4 != null)
        	{
        		var3 = this.worldObj.getWorldVec3Pool().getVecFromPool(var4.hitVec.xCoord, var4.hitVec.yCoord, var4.hitVec.zCoord);
        	}

        	Entity var5 = null;
        	List var6 = this.worldObj.getEntitiesWithinAABBExcludingEntity(this, this.boundingBox.addCoord(this.motionX, this.motionY, this.motionZ).expand(1.0D, 1.0D, 1.0D));
        	double var7 = 0.0D;
        	Iterator var9 = var6.iterator();
        	float var11;

        	while (var9.hasNext())
        	{
        		Entity var10 = (Entity)var9.next();
        		if (var10.canBeCollidedWith() && (var10 != this.shootingEntity || this.ticksInAir >= 5))
        		{
        			var11 = 0.3F;
        			AxisAlignedBB var12 = var10.boundingBox.expand((double)var11, (double)var11, (double)var11);
        			MovingObjectPosition var13 = var12.calculateIntercept(var17, var3);
        			if (var13 != null)
        			{
        				double var14 = var17.distanceTo(var13.hitVec);
        				if (var14 < var7 || var7 == 0.0D)
        				{
        					var5 = var10;
        					var7 = var14;
        				}
        			}
        		}
        	}

        	if (var5 != null)
        	{
        		var4 = new MovingObjectPosition(var5);
        	}

        	float var20;
        	if (var4 != null)
        	{
        		if (var4.entityHit != null)
        		{
        			this.motionX *= -0.10000000149011612D;
        			this.motionY *= -0.10000000149011612D;
        			this.motionZ *= -0.10000000149011612D;
        			this.rotationYaw += 180.0F;
        			this.prevRotationYaw += 180.0F;
        			this.ticksInAir = 0;
        		}
        		else
        		{
        			this.xTile = var4.blockX;
        			this.yTile = var4.blockY;
        			this.zTile = var4.blockZ;
        			this.inTile = this.worldObj.getBlockId(this.xTile, this.yTile, this.zTile);
        			this.inData = this.worldObj.getBlockMetadata(this.xTile, this.yTile, this.zTile);
        			this.motionX = (double)((float)(var4.hitVec.xCoord - this.posX));
        			this.motionY = (double)((float)(var4.hitVec.yCoord - this.posY));
        			this.motionZ = (double)((float)(var4.hitVec.zCoord - this.posZ));
        			var20 = MathHelper.sqrt_double(this.motionX * this.motionX + this.motionY * this.motionY + this.motionZ * this.motionZ);
        			this.posX -= this.motionX / (double)var20 * 0.05000000074505806D;
        			this.posY -= this.motionY / (double)var20 * 0.05000000074505806D;
        			this.posZ -= this.motionZ / (double)var20 * 0.05000000074505806D;
        			this.worldObj.playSoundAtEntity(this, "random.bowhit", 1.0F, 1.2F / (this.rand.nextFloat() * 0.2F + 0.9F));
        			this.inGround = true;
        			this.arrowShake = 7;
        		}
        	}

        	this.posX += this.motionX;
        	this.posY += this.motionY;
        	this.posZ += this.motionZ;
        	var20 = MathHelper.sqrt_double(this.motionX * this.motionX + this.motionZ * this.motionZ);
        	this.rotationYaw = (float)(Math.atan2(this.motionX, this.motionZ) * 180.0D / Math.PI);
        	for (this.rotationPitch = (float)(Math.atan2(this.motionY, (double)var20) * 180.0D / Math.PI); this.rotationPitch - this.prevRotationPitch < -180.0F; this.prevRotationPitch -= 360.0F)
        	{
        		;
        	}

        	while (this.rotationPitch - this.prevRotationPitch >= 180.0F)
        	{
        		this.prevRotationPitch += 360.0F;
        	}

        	while (this.rotationYaw - this.prevRotationYaw < -180.0F)
        	{
        		this.prevRotationYaw -= 360.0F;
        	}

        	while (this.rotationYaw - this.prevRotationYaw >= 180.0F)
        	{
        		this.prevRotationYaw += 360.0F;
        	}

        	this.rotationPitch = this.prevRotationPitch + (this.rotationPitch - this.prevRotationPitch) * 0.2F;
        	this.rotationYaw = this.prevRotationYaw + (this.rotationYaw - this.prevRotationYaw) * 0.2F;
        	float var23 = 0.99F;
        	var11 = 0.05F;
        	if (this.isInWater())
        	{
        		for (int var26 = 0; var26 < 4; ++var26)
        		{
        			float var27 = 0.25F;
        			this.worldObj.spawnParticle("bubble", this.posX - this.motionX * (double)var27, this.posY - this.motionY * (double)var27, this.posZ - this.motionZ * (double)var27, this.motionX, this.motionY, this.motionZ);
        		}
        		var23 = 0.8F;
        	}

        	this.motionX *= (double)var23;
        	this.motionY *= (double)var23;
        	this.motionZ *= (double)var23;
        	this.motionY -= (double)var11;
        	this.setPosition(this.posX, this.posY, this.posZ);
        	this.doBlockCollisions();
        }
        if (!this.worldObj.isRemote && this.worldObj.difficultySetting == 0)
        {
        	this.setDead();
        }
    }

    @Override
    protected void updateFallState(double par1, boolean par3)
    {
        if (par3)
        {
        	if (this.fallDistance > 0.0F)
        	{
        		this.fall(this.fallDistance);
        		this.fallDistance = 0.0F;
        	}
        }
        else if (par1 < 0.0D)
        {
        	this.fallDistance = (float)((double)this.fallDistance - par1);
        }
    }

    @Override
    public boolean canBePushed()
    {
        return false;
    }

    private boolean isValidLightLevel()
    {
        int var1 = MathHelper.floor_double(this.posX);
        int var2 = MathHelper.floor_double(this.boundingBox.minY);
        int var3 = MathHelper.floor_double(this.posZ);

        if (mod_QB.GSLightLevel <= 0)
        {
        	return false;
        }
        else if (this.worldObj.getSavedLightValue(EnumSkyBlock.Sky, var1, var2, var3) > this.rand.nextInt(32))
        {
        	return false;
        }
        else
        {
        	int var4 = this.worldObj.getBlockLightValue(var1, var2, var3);

        	if (this.worldObj.isThundering())
        	{
        		int var5 = this.worldObj.skylightSubtracted;
        		this.worldObj.skylightSubtracted = 10;
        		var4 = this.worldObj.getBlockLightValue(var1, var2, var3);
        		this.worldObj.skylightSubtracted = var5;
        	}

        	return var4 <= mod_QB.GSLightLevel;
        }
    }

    @Override
    public boolean getCanSpawnHere()
    {
        if (this.rand.nextInt(50) == 0)
        {
        	return false;
        }
        else if (this.worldObj.difficultySetting == 0)
        {
        	return false;
        }
        else if (!this.isValidLightLevel())
        {
        	return false;
        }
        else if (!this.worldObj.checkIfAABBIsClear(this.boundingBox))
        {
        	return false;
        }
        else if (!this.worldObj.getCollidingBoundingBoxes(this, this.boundingBox).isEmpty())
        {
        	return false;
        }
        else if (this.worldObj.isAnyLiquid(this.boundingBox))
        {
        	return false;
        }
        else
        {
        	return true;
        }
    }

    @Override
    public void writeEntityToNBT(NBTTagCompound par1NBTTagCompound)
    {
        par1NBTTagCompound.setShort("xTile", (short)this.xTile);
        par1NBTTagCompound.setShort("yTile", (short)this.yTile);
        par1NBTTagCompound.setShort("zTile", (short)this.zTile);
        par1NBTTagCompound.setByte("inTile", (byte)this.inTile);
        par1NBTTagCompound.setByte("inData", (byte)this.inData);
        par1NBTTagCompound.setByte("shake", (byte)this.arrowShake);
        par1NBTTagCompound.setByte("inGround", (byte)(this.inGround ? 1 : 0));
        par1NBTTagCompound.setInteger("countDown", this.getCountDown());
        par1NBTTagCompound.setInteger("SoulJem", this.getSoulGemDamage());
    }

    @Override
    public void readEntityFromNBT(NBTTagCompound par1NBTTagCompound)
    {
        this.xTile = par1NBTTagCompound.getShort("xTile");
        this.yTile = par1NBTTagCompound.getShort("yTile");
        this.zTile = par1NBTTagCompound.getShort("zTile");
        this.inTile = par1NBTTagCompound.getByte("inTile") & 255;
        this.inData = par1NBTTagCompound.getByte("inData") & 255;
        this.arrowShake = par1NBTTagCompound.getByte("shake") & 255;
        this.inGround = par1NBTTagCompound.getByte("inGround") == 1;
        this.dataWatcher.updateObject(21, par1NBTTagCompound.getInteger("countDown"));
        this.setSoulGemDamage(par1NBTTagCompound.getInteger("SoulJem"));
    }

    public int getSoulGemDamage()
    {
        return this.dataWatcher.getWatchableObjectInt(20);
    }

    public void setSoulGemDamage(int par1)
    {
        this.dataWatcher.updateObject(20, this.getSoulGemDamage() + par1);
    }

    public int getCountDown()
    {
        return this.dataWatcher.getWatchableObjectInt(21);
    }

    public void setNewCountDown()
    {
        this.dataWatcher.updateObject(21, 1000 + this.rand.nextInt(500) - this.getSoulGemDamage() * 15);
    }

    public void startCountDown()
    {
        this.dataWatcher.updateObject(21, this.getCountDown() - 1);
    }
}