package PuellaMagi.mods;

import net.minecraft.block.Block;
import net.minecraft.enchantment.Enchantment;
import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.ai.EntityAILookIdle;
import net.minecraft.entity.ai.EntityAISwimming;
import net.minecraft.entity.ai.EntityAIWander;
import net.minecraft.entity.ai.EntityAIWatchClosest;
import net.minecraft.entity.item.EntityFireworkRocket;
import net.minecraft.entity.item.EntityTNTPrimed;
import net.minecraft.entity.monster.EntityMob;
import net.minecraft.entity.monster.EntitySlime;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemDye;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.DamageSource;
import net.minecraft.util.MathHelper;
import net.minecraft.world.World;

public class EntityHomura extends EntityMahoShojo
{
    private int rapidFire;
    private int rapidFire2;
    private int rapidFire3;
    private ItemStack fireWork;

    public EntityHomura(World par1World)
    {
        super(par1World);
        this.texture = "/PuellaMagi/Mobs/homura.png";
        this.MSTexture = "/PuellaMagi/Mobs/homuraMS.png";
        this.attackStrength = 1;
        this.MSAttackStrength = 5;
        this.MSProtectStrength = 15;
        this.moveSpeed = 0.3F;
        this.soulGemID = mod_QB.homuraGem.shiftedIndex;
        this.rapidFire = this.rapidFire2 = 0;

        this.getNavigator().setAvoidsWater(true);
        this.tasks.addTask(1, new EntityAISwimming(this));
        this.tasks.addTask(2, new EntityMadomagiAIStandby(this));
        this.tasks.addTask(3, new EntityMadomagiAIShortRangeAttack(this, this.moveSpeed, 7.0F, new ItemStack(mod_QB.homuraGun1)));
        this.tasks.addTask(4, new EntityMadomagiAIMiddleRangeAttack(this, this.moveSpeed, 14.0F, new ItemStack(mod_QB.homuraGun2)));
        this.tasks.addTask(5, new EntityMadomagiAILongRangeAttack(this, this.moveSpeed, 15.0F, new ItemStack(Block.tnt)));
        this.tasks.addTask(6, new EntityMadomagiAIFollowOwner(this, this.moveSpeed, 1.5F));
        this.tasks.addTask(7, new EntityAIWander(this, this.moveSpeed));
        this.tasks.addTask(8, new EntityAIWatchClosest(this, EntityPlayer.class, 8.0F));
        this.tasks.addTask(8, new EntityAILookIdle(this));
        this.targetTasks.addTask(1, new EntityMadomagiAIOwnerHurtByTarget(this));
        this.targetTasks.addTask(2, new EntityMadomagiAIOwnerHurtTarget(this));
        this.targetTasks.addTask(3, new EntityMadomagiAINearestAttackableTarget(this, EntityMob.class, 16.0F, 4.0F, true));
        this.targetTasks.addTask(4, new EntityMadomagiAINearestAttackableTarget(this, EntitySlime.class, 16.0F, 4.0F, true));
    }

    @Override
    public int getMaxHealth()
    {
        return 12;
    }

    @Override
    public boolean attackEntityFrom(DamageSource par1DamageSource, int par2)
    {
        if (!this.worldObj.isRemote && !(par1DamageSource.getEntity() instanceof EntityPlayer) && this.isTransformation() && !par1DamageSource.isFireDamage())
        {
        	for (int var2 = 0; var2 < 64; var2++)
        	{
        		if (this.teleportRandomly())
        		{
        			if (this.isStandbyMode())
        			{
        				this.setFollowMode();
        			}
            		this.setSoulGemDamage(1);
        			return false;
        		}
        	}
        }
        return super.attackEntityFrom(par1DamageSource, par2);
    }

    private boolean teleportRandomly()
    {
        double var1 = this.posX + (this.rand.nextDouble() - 0.5D) * 12.0D;
        double var3 = this.posY + (double)(this.rand.nextInt(24) - 12);
        double var5 = this.posZ + (this.rand.nextDouble() - 0.5D) * 12.0D;
        return this.teleportTo(var1, var3, var5);
    }

    private boolean teleportTo(double par1, double par3, double par5)
    {
        double var7 = this.posX;
        double var9 = this.posY;
        double var11 = this.posZ;
        this.posX = par1;
        this.posY = par3;
        this.posZ = par5;
        boolean var13 = false;
        int var14 = MathHelper.floor_double(this.posX);
        int var15 = MathHelper.floor_double(this.posY);
        int var16 = MathHelper.floor_double(this.posZ);
        int var18;

        if (this.worldObj.blockExists(var14, var15, var16))
        {
        	boolean var17 = false;

        	while (!var17 && var15 > 0)
        	{
        		var18 = this.worldObj.getBlockId(var14, var15 - 1, var16);

        		if (var18 != 0 && Block.blocksList[var18].blockMaterial.blocksMovement())
        		{
        			var17 = true;
        		}
        		else
        		{
        			--this.posY;
        			--var15;
        		}
        	}

        	if (var17)
        	{
        		this.setPosition(this.posX, this.posY, this.posZ);

        		if (this.worldObj.getCollidingBoundingBoxes(this, this.boundingBox).isEmpty() && !this.worldObj.isAnyLiquid(this.boundingBox))
        		{
        			var13 = true;
        		}
        	}
        }

        if (!var13)
        {
        	this.setPosition(var7, var9, var11);
        	return false;
        }
        else
        {
        	short var30 = 128;
        	for (var18 = 0; var18 < var30; ++var18)
        	{
        		double var19 = (double)var18 / ((double)var30 - 1.0D);
        		float var21 = (this.rand.nextFloat() - 0.5F) * 0.2F;
        		float var22 = (this.rand.nextFloat() - 0.5F) * 0.2F;
        		float var23 = (this.rand.nextFloat() - 0.5F) * 0.2F;
        		double var24 = var7 + (this.posX - var7) * var19 + (this.rand.nextDouble() - 0.5D) * (double)this.width * 2.0D;
        		double var26 = var9 + (this.posY - var9) * var19 + this.rand.nextDouble() * (double)this.height;
        		double var28 = var11 + (this.posZ - var11) * var19 + (this.rand.nextDouble() - 0.5D) * (double)this.width * 2.0D;
        		this.worldObj.spawnParticle("portal", var24, var26, var28, (double)var21, (double)var22, (double)var23);
        	}
        	this.worldObj.playSoundEffect(var7, var9, var11, "mob.endermen.portal", 1.0F, 1.0F);
        	this.worldObj.playSoundAtEntity(this, "mob.endermen.portal", 1.0F, 1.0F);
        	return true;
        }
    }

    @Override
    public int shortRangeAttack(EntityLiving par1EntityLiving, int par2)
    {
        if (!this.isPosture())
        {
        	this.setPosture(true);
        }

        if (par2 <= 0)
        {
        	this.homuraShot(par1EntityLiving, 1.0F, 1.2F, 5, 0, 0);
        	if (this.rapidFire < 6)
        	{
        		this.rapidFire++;
        		return 5;
        	}
        	else
        	{
        		this.rapidFire = 0;
        		this.setSoulGemDamage(1);
        		return 20;
        	}
        }
        return par2;
    }

    @Override
    public int middleRangeAttack(EntityLiving par1EntityLiving, int par2)
    {
        if (!this.isPosture())
        {
        	this.setPosture(true);
        }

        if (par2 <= 0)
        {
        	this.homuraShot(par1EntityLiving, 1.6F, 12.0F, 2, 0, 0);
        	if (this.rapidFire2 < 30)
        	{
        		this.rapidFire2++;
        		return 2;
        	}
        	else
        	{
        		this.rapidFire2 = 0;
        		this.setSoulGemDamage(1);
        		return 20;
        	}
        }
        return par2;
    }

    @Override
    public int longRangeAttack(EntityLiving par1EntityLiving, int par2)
    {
        if (par2 <= 0)
        {
        	int var5 = (int)(par1EntityLiving.posX + (this.rand.nextFloat() - 0.5F) * this.width);
        	int var6 = (int)(par1EntityLiving.posY + this.rand.nextFloat() * this.height);
        	int var7 = (int)(par1EntityLiving.posZ + (this.rand.nextFloat() - 0.5F) * this.width);
        	if (this.worldObj.getBlockId(var5, var6, var7) == 0)
        	{
        		for (int var8 = 0; var8 < 64; var8++)
        		{
        			if (this.teleportRandomly())
        			{
        				EntityTNTPrimed var9 = new EntityTNTPrimed(this.worldObj, (double)var5 + 0.5D, (double)var6 + 0.5D, (double)var7 + 0.5D);
        				var9.fuse = var9.fuse / 8 + this.rand.nextInt(10) + this.rand.nextInt(10);
        				if (par1EntityLiving instanceof EntityWalpurgisnacht)
        				{
        					var9.fuse = 1;
        					this.worldObj.spawnEntityInWorld(var9);
        					ItemStack var10 = this.getFireWork();
        					EntityFireworkRocket var11 = new EntityFireworkRocket(this.worldObj, this.posX, this.posY, this.posZ, var10);
        					double var21 = par1EntityLiving.posX - this.posX;
        					double var23 = par1EntityLiving.posY + par1EntityLiving.getEyeHeight() - this.posY;
        					double var25 = par1EntityLiving.posZ - this.posZ;
        					double var27 = (double)MathHelper.sqrt_double(var21 * var21 + var23 + var23 + var25 * var25);
        					var11.motionX = var21 / var27 * 0.05D;
        					var11.motionY = var23 / var27 * 0.05D;
        					var11.motionZ = var25 / var27 * 0.05D;
        					this.worldObj.spawnEntityInWorld(var11);
        					if (this.rapidFire3 < 15)
        					{
        						this.rapidFire++;
        						return 1;
        					}
        					else
        					{
        						this.rapidFire = 0;
        						this.setSoulGemDamage(1);
        						return 100;
        					}
        				}
        				var9.motionX = par1EntityLiving.motionX;
        				var9.motionY = par1EntityLiving.motionY;
        				var9.motionZ = par1EntityLiving.motionZ;
        				this.worldObj.spawnEntityInWorld(var9);
        				this.worldObj.playSoundAtEntity(var9, "random.fuse", 1.0F, 1.0F);
                		this.setSoulGemDamage(1);
        				return var9.fuse;
        			}
        		}
        	}
        }
        return par2;
    }

    private void homuraShot(EntityLiving par1EntityLiving, float par2, float par3, int par4, int par5, int par6)
    {
        EntityMadomagiBullet var2 = new EntityMadomagiBullet(this.worldObj, this, par1EntityLiving, par2, par3);
        int var3 = EnchantmentHelper.getEnchantmentLevel(Enchantment.power.effectId, this.getHeldItem()) + par4;
        int var4 = EnchantmentHelper.getEnchantmentLevel(Enchantment.punch.effectId, this.getHeldItem()) + par5;
        int var5 = EnchantmentHelper.getEnchantmentLevel(Enchantment.flame.effectId, this.getHeldItem()) + par6;

        if (var3 > 0)
        {
        	var2.setDamagePower(var2.getDamagePower() + (double)var3 * 0.5D + 0.5D);
        }
        if (var4 > 0)
        {
        	var2.setKnockbackStrength(var4);
        }
        if (var5 > 0)
        {
        	var2.setFire(100);
        }

        this.worldObj.playSoundAtEntity(this, "random.bow", 1.0F, 1.0F / (this.getRNG().nextFloat() * 0.4F + 0.8F));
        this.worldObj.spawnEntityInWorld(var2);
    }

    private ItemStack getFireWork()
    {
        this.fireWork = new ItemStack(Item.field_92052_bU);
        if (!this.fireWork.hasTagCompound())
        {
        	this.fireWork.setTagCompound(new NBTTagCompound());
        }
        if (!this.fireWork.getTagCompound().hasKey("Fireworks"))
        {
        	this.fireWork.setTagCompound(new NBTTagCompound("Fireworks"));
        }
        if (!this.fireWork.getTagCompound().hasKey("Explosion"))
        {
        	this.fireWork.setTagCompound(new NBTTagCompound("Explosion"));
        }
        NBTTagCompound var15 = this.fireWork.getTagCompound();
        NBTTagCompound var18 = this.fireWork.getTagCompound().getCompoundTag("Fireworks");
        NBTTagCompound var20 = this.fireWork.getTagCompound().getCompoundTag("Explosion");
        NBTTagList var25 = new NBTTagList("Explosions");
        int[] var24 = new int[2];
        int[] var26 = new int[1];
        var24[0] = var26[0] = ItemDye.dyeColors[0];
        var24[1] = ItemDye.dyeColors[1];
        var20.setIntArray("Colors", var24);
        var20.setByte("Type", (byte)1);
        var20.setIntArray("FadeColors", var26);
        var25.appendTag(var20);
        var18.setTag("Explosions", var25);
        var18.setByte("Flight", (byte)2);
        var15.setTag("Fireworks", var18);
        this.fireWork.setTagCompound(var15);
        return this.fireWork;
    }
}