package PuellaMagi.mods;

import net.minecraft.enchantment.Enchantment;
import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.ai.EntityAILookIdle;
import net.minecraft.entity.ai.EntityAISwimming;
import net.minecraft.entity.ai.EntityAIWander;
import net.minecraft.entity.ai.EntityAIWatchClosest;
import net.minecraft.entity.item.EntityTNTPrimed;
import net.minecraft.entity.monster.EntityMob;
import net.minecraft.entity.passive.EntityAmbientCreature;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.util.DamageSource;
import net.minecraft.world.World;

public class EntityMadoka extends EntityMahoShojo
{
    private int rapidFire;
    private int rapidFire2;

    public EntityMadoka(World par1World)
    {
        super(par1World);
        this.texture = "/PuellaMagi/Mobs/madoka.png";
        this.MSTexture = "/PuellaMagi/Mobs/madokaMS.png";
        this.attackStrength = 1;
        this.MSAttackStrength = 5;
        this.MSProtectStrength = 10;
        this.moveSpeed = 0.3F;
        this.soulGemID = mod_QB.madokaGem.shiftedIndex;
        this.majo = new EntityKriemhildGretchen(par1World);
        this.rapidFire = this.rapidFire2 = 0;
        ItemStack var1 = new ItemStack(mod_QB.madokaWeapon);

        this.getNavigator().setAvoidsWater(true);
        this.tasks.addTask(1, new EntityAISwimming(this));
        this.tasks.addTask(2, new EntityMadomagiAIAvoidEntity(this, EntityTNTPrimed.class, 16.0F, this.moveSpeed, this.moveSpeed + 0.2F));
        this.tasks.addTask(3, new EntityMadomagiAIStandby(this));
        this.tasks.addTask(4, new EntityMadomagiAIShortRangeAttack(this, this.moveSpeed, 5.0F, var1));
        this.tasks.addTask(5, new EntityMadomagiAIMiddleRangeAttack(this, this.moveSpeed, 10.0F, var1));
        this.tasks.addTask(6, new EntityMadomagiAILongRangeAttack(this, this.moveSpeed, 16.0F, var1));
        this.tasks.addTask(7, new EntityMadomagiAIFollowOwner(this, this.moveSpeed, 1.0F));
        this.tasks.addTask(8, new EntityAIWander(this, this.moveSpeed));
        this.tasks.addTask(9, new EntityAIWatchClosest(this, EntityPlayer.class, 8.0F));
        this.tasks.addTask(9, new EntityAILookIdle(this));
        this.targetTasks.addTask(1, new EntityMadomagiAIOwnerHurtByTarget(this));
        this.targetTasks.addTask(2, new EntityMadomagiAIOwnerHurtTarget(this));
        this.targetTasks.addTask(3, new EntityMadomagiAINearestAttackableTarget(this, EntityMob.class, 16.0F, 4.0F, true));
        this.targetTasks.addTask(4, new EntityMadomagiAINearestAttackableTarget(this, EntityAmbientCreature.class, 16.0F, 4.0F, true));
    }

    @Override
    public int getMaxHealth()
    {
        return 16;
    }

    @Override
    public boolean attackEntityFrom(DamageSource par1DamageSource, int par2)
    {
        if (!this.worldObj.isRemote && par1DamageSource.getEntity() == this && this.isTransformation())
        {
        	return false;
        }
        return super.attackEntityFrom(par1DamageSource, par2);
    }

    @Override
    public int shortRangeAttack(EntityLiving par1EntityLiving, int par2)
    {
        if (!this.isPosture())
        {
        	this.setPosture(true);
        }

        if (par2 <= 0)
        {
        	for (int var1 = 0; var1 < 5; var1++)
        	{
        		this.madokaShooting(par1EntityLiving, 3.6F, 60.0F, 0, 1, 0);
        	}
        	this.setSoulGemDamage(1);
        	return 20;
        }
        return par2;
    }

    @Override
    public int middleRangeAttack(EntityLiving par1EntityLiving, int par2)
    {
        if (!this.isPosture())
        {
        	this.setPosture(true);
        }

        if (par2 <= 0)
        {
        	this.madokaShooting(par1EntityLiving, 1.6F, 6.0F, 5, 0, 0);
        	if (this.rapidFire < 7)
        	{
        		this.rapidFire++;
        		return 3;
        	}
        	else
        	{
        		this.setSoulGemDamage(1);
        		this.rapidFire = 0;
        		return 10;
        	}
        }
        return par2;
    }

    @Override
    public int longRangeAttack(EntityLiving par1EntityLiving, int par2)
    {
        if (!this.isPosture())
        {
        	this.setPosture(true);
        }

        if (par2 <= 0)
        {
        	float var2 = par1EntityLiving instanceof EntityWalpurgisnacht ? 4.8F : 1.6F;
        	for (int var1 = 0; var1 < 4; var1++)
        	{
        		this.madokaShooting(par1EntityLiving, var2, 12.0F, 3, 0, 0);
        	}

        	if (this.rapidFire2 < 4)
        	{
        		this.rapidFire2++;
        		return 3;
        	}
        	else
        	{
        		this.rapidFire2 = 0;
        		this.setSoulGemDamage(1);
        		return 15;
        	}
        }
        return par2;
    }

    private void madokaShooting(EntityLiving par1EntityLiving, float par2, float par3, int par4, int par5, int par6)
    {
        EntityLightArrow var2 = new EntityLightArrow(this.worldObj, this, par1EntityLiving, par2, par3);
        int var3 = EnchantmentHelper.getEnchantmentLevel(Enchantment.power.effectId, this.getHeldItem()) + par4;
        int var4 = EnchantmentHelper.getEnchantmentLevel(Enchantment.punch.effectId, this.getHeldItem()) + par5;
        int var5 = EnchantmentHelper.getEnchantmentLevel(Enchantment.flame.effectId, this.getHeldItem()) + par6;

        if (var3 > 0)
        {
        	var2.setDamagePower(var2.getDamagePower() + (double)var3 * 0.5D + 0.5D);
        }
        if (var4 > 0)
        {
        	var2.setKnockbackStrength(var4);
        }
        if (var5 > 0)
        {
        	var2.setFire(100);
        }

        this.worldObj.playSoundAtEntity(this, "random.bow", 1.0F, 1.0F / (this.getRNG().nextFloat() * 0.4F + 0.8F));
        this.worldObj.spawnEntityInWorld(var2);
    }
}