package PuellaMagi.mods;

import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityCreature;
import net.minecraft.entity.passive.IAnimals;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.potion.Potion;
import net.minecraft.util.DamageSource;
import net.minecraft.util.MathHelper;
import net.minecraft.world.World;

public abstract class EntityMadomagi extends EntityCreature implements IAnimals
{
    protected int attackStrength;
    protected int protectSterngth;
    protected int dropItemID;
    protected String lastMsg;
    protected int healTime;
    protected int healFrequency;

    public EntityMadomagi(World par1World)
    {
        super(par1World);
        this.setSize(0.6F, 1.8F);
        this.attackStrength = 2;
        this.protectSterngth = 0;
        this.dropItemID = 0;
        this.lastMsg = "";
        this.healTime = 0;
        this.healFrequency = 100;
    }

    @Override
    public void onLivingUpdate()
    {
        this.autoHealing();
        if ((this.isInWater() || !this.onGround) && this.isStandbyMode() && this.isEnable())
        {
        	this.setFollowMode();
        }
        super.onLivingUpdate();
    }

    @Override
    public boolean isAIEnabled()
    {
        return true;
    }

    @Override
    public float getEyeHeight()
    {
        return this.height * 0.8F;
    }

    @Override
    protected boolean canDespawn()
    {
        return !this.isEnable();
    }

    @Override
    protected float getSoundVolume()
    {
        return 1.0F;
    }

    @Override
    public boolean attackEntityFrom(DamageSource par1DamageSource, int par2)
    {
        Entity var3 = par1DamageSource.getEntity();
        if (var3 != null && !(var3 instanceof EntityPlayer))
        {
        	par2 = MathHelper.floor_double((double)(par2 * (100 - this.getProtectPower()) / 100));
        }
        return super.attackEntityFrom(par1DamageSource, par2);
    }

    public int getProtectPower()
    {
        return this.protectSterngth;
    }

    @Override
    public void damageEntity(DamageSource par1DamageSource, int par2)
    {
        super.damageEntity(par1DamageSource, par2);
        if (this.getHealth() > 0)
        {
        	this.chatMessage("HP = " + this.getHealth() * 0.5);
        }
    }

    @Override
    public boolean attackEntityAsMob(Entity par1Entity)
    {
        int var2 = this.getAttackPower();

        if (this.isPotionActive(Potion.damageBoost))
        {
        	var2 += 3 << this.getActivePotionEffect(Potion.damageBoost).getAmplifier();
        }

        if (this.isPotionActive(Potion.weakness))
        {
        	var2 -= 2 <<this.getActivePotionEffect(Potion.weakness).getAmplifier();
        }

        return par1Entity.attackEntityFrom(DamageSource.causeMobDamage(this), var2);
    }

    public int getAttackPower()
    {
        return this.attackStrength;
    }

    @Override
    protected void entityInit()
    {
        super.entityInit();
        this.dataWatcher.addObject(15, Byte.valueOf((byte)0));
        this.dataWatcher.addObject(16, Byte.valueOf((byte)-1));
        this.dataWatcher.addObject(17, "");
    }

    @Override
    public void writeEntityToNBT(NBTTagCompound par1NBTTagCompound)
    {
        super.writeEntityToNBT(par1NBTTagCompound);
        if (this.getOwnerName() == null)
        {
        	par1NBTTagCompound.setString("Owner", "");
        }
        else
        {
        	par1NBTTagCompound.setString("Owner", this.getOwnerName());
        }
    }

    @Override
    public void readEntityFromNBT(NBTTagCompound par1NBTTagCompound)
    {
        super.readEntityFromNBT(par1NBTTagCompound);
        this.setStandbyMode();
        String var1 = par1NBTTagCompound.getString("Owner");
        if (var1.length() > 0)
        {
        	this.setOwner(var1);
        	if (this.getOwner() != null)
        	{
        		this.setEnable();
        	}
        }
    }

    @Override
    public int getVerticalFaceSpeed()
    {
        return this.isStandbyMode() ? 20 : super.getVerticalFaceSpeed();
    }

    @Override
    public boolean interact(EntityPlayer par1EntityPlayer)
    {
        ItemStack var2 = par1EntityPlayer.inventory.getCurrentItem();
        if (var2 != null && var2.itemID == mod_QB.madokaRibbon.shiftedIndex)
        {
        	if (!this.worldObj.isRemote)
        	{
        		if (this.isEnable())
        		{
        			this.changeMode();
        		}
        		else
        		{
        			this.setEnable();
        			this.setPathToEntity(null);
        			this.setAttackTarget(null);
        			this.setOwner(par1EntityPlayer.username);
        			this.chatMessage("Awaken!");
        			this.setStandbyMode();
        		}
        	}
        	return true;
        }
        return super.interact(par1EntityPlayer);
    }

    @Override
    public void onDeath(DamageSource par1DamageSource)
    {
        super.onDeath(par1DamageSource);
        this.dataWatcher.updateObject(16, Byte.valueOf((byte)-1));
        this.chatMessage("logout");
    }

    @Override
    protected void dropFewItems(boolean par1, int par2)
    {
        if (this.dropItemID > 0)
        {
        	this.dropItem(this.dropItemID, 1);
        }
    }

    public String getOwnerName()
    {
        return this.dataWatcher.getWatchableObjectString(17);
    }

    public void setOwner(String par1Str)
    {
        this.dataWatcher.updateObject(17, par1Str);
    }

    public EntityPlayer getOwner()
    {
        return this.worldObj.getPlayerEntityByName(getOwnerName());
    }

    protected void chatMessage(String par1Str)
    {
        EntityPlayer var1EntityPlayer = this.getOwner();
        if (var1EntityPlayer != null && !this.worldObj.isRemote)
        {
        	if (!this.lastMsg.equalsIgnoreCase(par1Str))
        	{
        		var1EntityPlayer.addChatMessage(" " + this.getEntityString() + " : " + par1Str);
        		this.lastMsg = par1Str;
        	}
        	System.out.println(" " + this.getEntityString() + " : " + par1Str);
        }
    }

    public boolean isStandbyMode()
    {
        return this.dataWatcher.getWatchableObjectByte(16) == 0;
    }

    public void setStandbyMode()
    {
        this.dataWatcher.updateObject(16, Byte.valueOf((byte)0));
        this.mountEntity(null);
        this.setPathToEntity(null);
        this.chatMessage("Standby mode");
        this.worldObj.playSoundEffect(this.posX, this.posY +(double)this.getEyeHeight(), this.posZ, "random.click", 0.3F, 0.6F);
    }

    public boolean isFreeMode()
    {
        return this.dataWatcher.getWatchableObjectByte(16) == 1;
    }

    public void setFreeMode()
    {
        this.dataWatcher.updateObject(16, Byte.valueOf((byte)1));
        this.mountEntity(null);
        this.chatMessage("Free mode");
        this.worldObj.playSoundEffect(this.posX, this.posY + (double)this.getEyeHeight(), this.posZ, "random.click", 0.3F, 0.6F);
    }

    public boolean isFollowMode()
    {
        return this.dataWatcher.getWatchableObjectByte(16) == 2;
    }

    public void setFollowMode()
    {
        this.dataWatcher.updateObject(16, Byte.valueOf((byte)2));
        this.mountEntity(null);
        this.chatMessage("Follow mode");
        this.worldObj.playSoundEffect(this.posX, this.posY + (double)this.getEyeHeight(), this.posZ, "random.click", 0.3F, 0.6F);
    }

    public void changeMode()
    {
        String var1;
        if (this.isStandbyMode())
        {
        	this.setFreeMode();
        	var1 = "note";
        }
        else if (this.isFreeMode())
        {
        	this.setFollowMode();
        	var1 = "heart";
        }
        else
        {
        	this.setStandbyMode();
        	var1 = "spell";
        }

        double var0 = rand.nextGaussian() * 0.02D;
        double var2 = rand.nextGaussian() * 0.02D;
        double var4 = rand.nextGaussian() * 0.02D;
        this.worldObj.spawnParticle(var1, this.posX + (double)(rand.nextFloat() * this.width * 2.0F) - (double)this.width, this.posY + 0.5D + (double)(rand.nextFloat() * this.height), this.posZ + (double)(rand.nextFloat() * this.width * 2.0F) - (double)this.width, var0, var2, var4);

    }

    public boolean isEnable()
    {
        return this.dataWatcher.getWatchableObjectByte(15) == 1;
    }

    public void setEnable()
    {
        this.dataWatcher.updateObject(15, Byte.valueOf((byte)1));
    }

    public void autoHealing()
    {
        this.healTime++;
        if (this.healTime >= this.getHealFrequency())
        {
        	this.healTime = 0;
        	if (this.getHealth() < this.getMaxHealth())
        	{
        		this.heal(1);
        		if (this.getHealth() > 0)
        		{
        			this.chatMessage("HP = " + this.getHealth() * 0.5);
        		}
        	}
        }
    }

    protected int getHealFrequency()
    {
        return this.healFrequency;
    }

    @Override
    protected void dropEquipment(boolean par1, int par2){}

    public boolean canAttackExplosive()
    {
        return false;
    }
}