package PuellaMagi.mods;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.monster.EntityMob;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.world.ChunkPosition;
import net.minecraft.world.World;

public abstract class EntityMahoShojo extends EntityMadomagi
{
    protected String MSTexture;
    protected int MSAttackStrength;
    protected int MSProtectStrength;
    protected int transformTime;
    protected int transformLimit;
    protected int soulGemID;
    protected EntityMob majo;
    private int actionTick;
    private List teritoryBlockPositions = new ArrayList();

    public EntityMahoShojo(World par1World)
    {
        super(par1World);
        this.MSAttackStrength = 4;
        this.MSProtectStrength = 0;
        this.transformTime = 0;
        this.transformLimit = 500;
        this.soulGemID = 0;
        this.majo = new EntityGertrud(par1World);
        this.actionTick = 0;
    }

    @Override
    public String getTexture()
    {
        return this.isTransformation() ? this.MSTexture : super.getTexture();
    }

    @Override
    public void fall(float par1)
    {
        if (this.isNormal())
        {
        	super.fall(par1);
        }
    }

    @Override
    public void onLivingUpdate()
    {
        if (!this.worldObj.isRemote)
        {
        	if (this.isEnable())
        	{
        		if (this.isTransformation())
        		{
        			if (this.getAttackTarget() == null)
        			{
        				this.transformTime++;
        				if (this.transformTime > this.transformLimit)
        				{
        					this.transformTime = 0;
        					this.setNormal();
        				}
        			}
        			else if (this.transformTime > 0)
        			{
        				this.transformTime--;
        			}
        		}
        		else if (this.isNormal())
        		{
        			if (this.getAttackTarget() != null && !this.isStandbyMode() && this.getSoulGemDamage() < 56)
        			{
        				this.setTrancformation();
        			}
        		}
        	}
        }
        else if (this.getSoulGemDamage() >= 48)
        {
        	this.worldObj.spawnParticle("reddust", this.posX + (this.rand.nextDouble() - 0.5D) * (double)this.width, this.posY + this.rand.nextDouble() * (double)this.height - 0.25D, this.posZ + (this.rand.nextDouble() - 0.5D) * (double)this.width, 0.01D, 0.001D, 0.001D);
        	if (this.getSoulGemDamage() >= 56)
        	{
        		this.worldObj.spawnParticle("reddust", this.posX + (this.rand.nextDouble() - 0.5D) * (double)this.width * 1.2D, this.posY + this.rand.nextDouble() * (double)this.height + 0.25D, this.posZ + (this.rand.nextDouble() - 0.5D) * (double)this.width * 1.2D, 0.01D, 0.001D, 0.001D);
        	}
        }
        if (this.getSoulGemDamage() >= 64)
        {
        	this.spawnMajo();
        }
        super.onLivingUpdate();
    }

    @Override
    public int getProtectPower()
    {
        return this.isTransformation() ? this.MSProtectStrength : super.getProtectPower();
    }

    @Override
    public int getAttackPower()
    {
        return this.isTransformation() ? this.MSAttackStrength : super.getAttackPower();
    }

    @Override
    protected void entityInit()
    {
        super.entityInit();
        this.dataWatcher.addObject(18, Byte.valueOf((byte)0));
        this.dataWatcher.addObject(19, Byte.valueOf((byte)0));
        this.dataWatcher.addObject(20, new Integer(0));
    }

    @Override
    public void writeEntityToNBT(NBTTagCompound par1NBTTagCompound)
    {
        super.writeEntityToNBT(par1NBTTagCompound);
        par1NBTTagCompound.setByte("Trans", this.dataWatcher.getWatchableObjectByte(18));
        par1NBTTagCompound.setInteger("SoulJem", this.getSoulGemDamage());
    }

    @Override
    public void readEntityFromNBT(NBTTagCompound par1NBTTagCompound)
    {
        super.readEntityFromNBT(par1NBTTagCompound);
        this.dataWatcher.updateObject(18, par1NBTTagCompound.getByte("Trans"));
        this.setSoulGemDamage(par1NBTTagCompound.getInteger("SoulJem"));
    }

    @Override
    public boolean interact(EntityPlayer par1EntityPlayer)
    {
        ItemStack var1 = par1EntityPlayer.inventory.getCurrentItem();
        if (var1 != null && this.isEnable())
        {
        	if (this.isNormal() && var1.itemID == Item.appleGold.shiftedIndex)
        	{
        		if (!par1EntityPlayer.capabilities.isCreativeMode)
        		{
        			--var1.stackSize;
        		}

        		if (var1.stackSize <= 0)
        		{
        			par1EntityPlayer.inventory.setInventorySlotContents(par1EntityPlayer.inventory.currentItem, (ItemStack)null);
        		}

        		if (!this.worldObj.isRemote)
        		{
        			this.setTrancformation();
        		}
        		return true;
        	}
        	else if (this.getSoulGemDamage() > 0 && var1.itemID == mod_QB.itemGriefSeed.shiftedIndex)
        	{
        		if (!this.worldObj.isRemote)
        		{
        			int var2 = var1.getMaxDamage() - var1.getItemDamage();
        			var2 = Math.min(var2, 16);
        			var1.damageItem(this.healSoulGem(var2), par1EntityPlayer);
        		}
        		if (this.getSoulGemDamage() == 0)
        		{
        			this.chatMessage("All Darkness of Soul Gem is absorbed!!");
        		}
        		else
        		{
        			this.chatMessage("Darkness of Soul Gem is absorbed!");
        		}
        		return true;
        	}
        }
    	return super.interact(par1EntityPlayer);
    }

    @Override
    protected void dropFewItems(boolean par1, int par2)
    {
        if (this.soulGemID > 0)
        {
        	ItemStack var1 = new ItemStack(this.soulGemID, 1, this.getSoulGemDamage());
        	this.entityDropItem(var1, 0.0F);
        }
    }

    public void spawnMajo()
    {
        if (this.worldObj.difficultySetting == 0)
        {
        	this.chatMessage("She has gone, guided by the Law of the Circle.");
        	this.setDead();
        	return;
        }
        EntityMob var0 = this.majo;
        var0.setPositionAndRotation(this.posX, this.posY, this.posZ, this.rotationPitch, 0.0F);
        int var1 = 0;
        int var10 = (int)var0.height;
        HashSet var11 = new HashSet();
        while (true)
        {
        	if (var0.getCanSpawnHere() || var1 * 2 > var10)
        	{
        		if (!this.worldObj.isRemote)
        		{
        			this.worldObj.spawnEntityInWorld(var0);
        		}
        		var0.spawnExplosionParticle();
        		this.chatMessage("She changed the Whitch.");
        		this.setDead();
        		return;
        	}
        	for (int var2 = -var1; var2 <= var1; var2++)
        	{
        		for (int var3 = 0; var3 <= var1 * 2; var3++)
        		{
        			for (int var4 = -var1; var4 <= var1; var4++)
        			{
        				if (var2 == -var1 || var2 == var1 || var3 == var1 * 2 - 1 || var3 == var1 * 2 || var4 == -var1 || var4 == var1)
        				{
        					int var5 = (int)this.posX + var2;
        					int var6 = (int)this.posY + var3;
        					int var7 = (int)this.posZ + var4;
        					var11.add(new ChunkPosition(var5, var6, var7));
        				}
        			}
        		}
        	}
        	this.teritoryBlockPositions.addAll(var11);
        	this.breakTeritoryBlock();
        	var1++;
        }
    }

    private void breakTeritoryBlock()
    {
        Iterator var1 = this.teritoryBlockPositions.iterator();
        while (var1.hasNext())
        {
        	ChunkPosition var2 = (ChunkPosition)var1.next();
        	int var5 = var2.x;
        	int var6 = var2.y;
        	int var7 = var2.z;
        	int var8 = this.worldObj.getBlockId(var5, var6, var7);
        	if (var8 > 0 && var8 != Block.bedrock.blockID)
        	{
        		if (this.worldObj.setBlockWithNotify(var5, var6, var7, 0))
        		{
        			Block var9 = Block.blocksList[var8];
        			if (var8 == Block.dirt.blockID || var8 == Block.stone.blockID || var8 == Block.sand.blockID)
        			{
        				var9.dropBlockAsItemWithChance(this.worldObj, var5, var6, var7, this.worldObj.getBlockMetadata(var5, var6, var7), 0.001F, 0);
        			}
        			else
        			{
        				var9.dropBlockAsItemWithChance(this.worldObj, var5, var6, var7, this.worldObj.getBlockMetadata(var5, var6, var7), 0.1F, 0);
        			}
        			if (this.worldObj.setBlockAndMetadataWithUpdate(var5, var6, var7, 0, 0, this.worldObj.isRemote))
        			{
        				this.worldObj.notifyBlocksOfNeighborChange(var5, var6, var7, 0);
        			}
        		}
        	}
        }
        this.teritoryBlockPositions.clear();
    }

    public boolean isNormal()
    {
        return this.dataWatcher.getWatchableObjectByte(18) == 0;
    }

    public void setNormal()
    {
        this.dataWatcher.updateObject(18, Byte.valueOf((byte)0));
        this.mountEntity(null);
        this.setPathToEntity(null);
        this.chatMessage("Finish Transformation!");
        this.worldObj.playSoundEffect(this.posX, this.posY +(double)this.getEyeHeight(), this.posZ, "random.click", 0.3F, 0.6F);
    }

    public boolean isTransformation()
    {
        return this.dataWatcher.getWatchableObjectByte(18) == 1;
    }

    public void setTrancformation()
    {
        this.dataWatcher.updateObject(18, Byte.valueOf((byte)1));
        this.mountEntity(null);
        this.setPathToEntity(null);
        this.chatMessage("Transformation!");
        this.worldObj.playSoundEffect(this.posX, this.posY +(double)this.getEyeHeight(), this.posZ, "random.click", 0.3F, 0.6F);
        this.setSoulGemDamage(1);
    }

    public boolean isUltimateForm()
    {
        return this.dataWatcher.getWatchableObjectByte(18) == 2;
    }

    public void setUltimateForm()
    {
        this.dataWatcher.updateObject(18, Byte.valueOf((byte)2));
        this.mountEntity(null);
        this.setPathToEntity(null);
        this.chatMessage("Ultimet Form!");
        this.worldObj.playSoundEffect(this.posX, this.posY +(double)this.getEyeHeight(), this.posZ, "random.click", 0.3F, 0.6F);
    }

    public boolean isPosture()
    {
        return this.dataWatcher.getWatchableObjectByte(19) == 1;
    }

    public void setPosture(boolean par1)
    {
        if (par1)
        {
        	this.dataWatcher.updateObject(19, Byte.valueOf((byte)1));
        }
        else
        {
        	this.dataWatcher.updateObject(19, Byte.valueOf((byte)0));
        }
    }

    public int getSoulGemDamage()
    {
        return this.dataWatcher.getWatchableObjectInt(20);
    }

    public void setSoulGemDamage(int par1)
    {
        if (!this.worldObj.isRemote)
        {
        	this.dataWatcher.updateObject(20, this.getSoulGemDamage() + par1);
        }
    }

    public int healSoulGem(int par1)
    {
        int var1 = this.getSoulGemDamage();
        int var2 = Math.max(var1 - par1, 0);
        this.dataWatcher.updateObject(20, var2);
        return var2 == 0 ? par1 - var1 : par1;
    }

    @Override
    public boolean canAttackExplosive()
    {
        return this.isTransformation() ? true : this.getSoulGemDamage() < 32;
    }

    public abstract int shortRangeAttack(EntityLiving par1EntityLiving, int par2);
    public abstract int middleRangeAttack(EntityLiving par1EntityLiving, int par2);
    public abstract int longRangeAttack(EntityLiving par1EntityLiving, int par2);

    public void enchantWeapon()
    {
        this.func_82162_bC();
    }

    public int getActionTick()
    {
        return this.actionTick;
    }

    public void setActionTick(int par1)
    {
        this.actionTick = par1;
    }
}