package PuellaMagi.mods;

import java.util.LinkedList;

import net.minecraft.block.Block;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.ai.EntityAIBase;
import net.minecraft.entity.item.EntityTNTPrimed;
import net.minecraft.util.MathHelper;
import net.minecraft.world.World;

public class EntityMajoAIWalpurgisnachtAttack extends EntityAIBase
{
    private EntityWalpurgisnacht entityHost;
    private EntityLiving entityTarget;
    private double xPosition;
    private double yPosition;
    private double zPosition;
    private float moveSpeed;
    private int moveTick;
    private int attackTick;
    private LinkedList TNTsettingList;
    private int xTarget;
    private int yTarget;
    private int zTarget;
    private int attackTime1;
    private int attackTime2;

    public EntityMajoAIWalpurgisnachtAttack(EntityWalpurgisnacht par1EntityWalpurgisnacht, float par2)
    {
        this.entityHost = par1EntityWalpurgisnacht;
        this.moveSpeed = par2;
        this.TNTsettingList = new LinkedList();
        this.attackTick = 500;
        this.setMutexBits(3);
    }

    @Override
    public boolean shouldExecute()
    {
        EntityLiving var1 = this.entityHost.getAttackTarget();
        if (var1 == null)
        {
        	return false;
        }
        else if (this.entityHost.getHealth() > 15)
        {
        	return false;
        }
        else
        {
        	this.entityTarget = var1;
        	return true;
        }
    }

    @Override
    public void startExecuting()
    {
        this.initPosition();
        this.initMovement();
        this.moveTick = 1000;
        this.attackTime1 = 15 + this.entityHost.getRNG().nextInt(15);
        this.attackTime2 = 50 + this.entityHost.getRNG().nextInt(50);
    }

    @Override
    public void updateTask()
    {
        double var1 = this.entityHost.getDistanceSq(this.xPosition, this.yPosition, this.zPosition);
        if (var1 < 4.0D || this.moveTick-- < 0)
        {
        	this.initPosition();
        	this.moveTick = 1000;
        }
        this.initMovement();
        double var2 = this.entityTarget.posX - this.entityHost.posX;
        double var3 = this.entityTarget.posZ - this.entityHost.posZ;
        this.entityHost.rotationYaw = (float)(Math.atan2(var3, var2) * 180.0D / Math.PI) - 90.0F;
        if (++this.attackTick > 500)
        {
        	this.attackTick = 0;
        	this.attackDestroy();
        }
        if (this.attackTick % this.attackTime1 == 0)
        {
        	this.entityHost.attackFlameLance(this.entityTarget, this.entityHost.getRNG().nextInt(3) + 1);
            this.attackTime1 = 10 + this.entityHost.getRNG().nextInt(20);
        }
        if (this.attackTick % this.attackTime2 == 0)
        {
        	this.entityHost.attackPrickle(this.entityTarget, this.entityHost.getRNG().nextInt(3) + 1);
            this.attackTime2 = 50 + this.entityHost.getRNG().nextInt(50);
        }
    }

    private void initPosition()
    {
        double var1;
        do
        {
        	this.xPosition = this.entityTarget.posX + (double)(this.entityHost.getRNG().nextFloat() - 0.5F) * 100.0D;
        	this.zPosition = this.entityTarget.posZ + (double)(this.entityHost.getRNG().nextFloat() - 0.5F) * 100.0D;
        	this.yPosition = this.entityTarget.posY + (double)(this.entityHost.getRNG().nextFloat() - 0.1F) * 30.0D;
        	var1 = Math.min(this.entityHost.getDistanceSq(this.xPosition, this.yPosition, this.zPosition), this.entityTarget.getDistanceSq(this.xPosition, this.yPosition, this.zPosition));
        }
        while (var1 < 225.0D);
    }

    private void initMovement()
    {
        double var1 = this.entityHost.posX - this.xPosition;
        double var2 = this.entityHost.posY - this.yPosition;
        double var3 = this.entityHost.posZ - this.zPosition;
        double var4 = MathHelper.sqrt_double(var1 * var1 + var2 * var2 + var3 * var3);
        this.entityHost.motionX = -var1 * (float)this.moveSpeed / var4;
        this.entityHost.motionY = -var2 * (float)this.moveSpeed / var4;
        this.entityHost.motionZ = -var3 * (float)this.moveSpeed / var4;
    }

    private void attackDestroy()
    {
        this.xTarget = (int)this.entityTarget.posX;
        this.yTarget = (int)this.entityTarget.posY - 2;
        this.zTarget = (int)this.entityTarget.posZ;
        this.setTNT(this.entityHost.worldObj, this.xTarget, this.yTarget, this.zTarget);
    }

    public void setTNT(World par2World, int par4, int par5, int par6)
    {
        this.checkConnection(par2World, par4, par5, par6);
        while(this.setTNTBlock(par2World));
        this.TNTsettingList.clear();
    }

    public void checkConnection(World par2World, int par4, int par5, int par6)
    {
        int var0 = 1;
        int var9 = 1;
        int var2 = 1;
        int var11 = 1;
        int var4 = 1;
        int var13 = 1;

        if (this.xTarget + 3 == par4)
        {
        	var0 = 0;
        }

        if (this.xTarget - 3 == par4)
        {
        	var9 = 0;
        }

        var2 = 0;

        if (this.yTarget - 1 == par5)
        {
        	var11 = 0;
        }

        if (this.zTarget + 3 == par6)
        {
        	var4 = 0;
        }

        if (this.zTarget - 3 == par6)
        {
        	var13 = 0;
        }

        for (int var1 = -var9; var1 <= var0; var1++)
        {
        	for (int var3 = -var11; var3 <= var2; var3++)
        	{
        		for (int var5 = -var13; var5 <= var4; var5++)
        		{
        			int var6 = par2World.getBlockId(par4 + var1, par5 + var3, par6 + var5);
        			if (Block.blocksList[var6] != null)
        			{
        				int var8[] = {par4 + var1, par5 + var3, par6 + var5};
        				this.TNTsettingList.offer(var8);
        			}
        		}
        	}
        }
    }

    public boolean checkBlock(Block par1Block, World par2World, int par2, int par3, int par4)
    {
        if (par1Block == null || par1Block == Block.bedrock)
        {
        	return false;
        }

        for (int var1 = -1; var1 <= 1; var1++)
        {
        	for (int var3 = -1; var3 <= 1; var3++)
        	{
        		for (int var5 = -1; var5 <= 1; var5++)
        		{
        			int var8 = Math.abs(var1) + Math.abs(var3) + Math.abs(var5);
        			if (var8 <= 1)
        			{
        				int var6 = par2World.getBlockId(par2 + var1, par3 + var3, par4 + var5);
        				if (Block.blocksList[var6] == null)
        				{
        					return false;
        				}
        			}
        		}
        	}
        }

        return true;
    }

    public boolean setTNTBlock(World par2World)
    {
        if (this.TNTsettingList.size() == 0)
        {
        	return false;
        }
        if (this.TNTsettingList.peek() == null)
        {
        	return false;
        }
        int var1[] = (int[])this.TNTsettingList.poll();
        if (var1 == null)
        {
        	return false;
        }

        Block var2Block = Block.blocksList[par2World.getBlockId(var1[0], var1[1], var1[2])];
        if (this.checkBlock(var2Block, par2World, var1[0], var1[1], var1[2]))
        {
        	if (par2World.setBlockWithNotify(var1[0], var1[1], var1[2], 0))
        	{
        		EntityTNTPrimed var5 = new EntityTNTPrimed(par2World, var1[0], var1[1], var1[2]);
        		var5.fuse = 100;
        		var5.motionX = var5.motionY = var5.motionZ = 0.0F;
        		par2World.spawnEntityInWorld(var5);
        		par2World.playSoundAtEntity(var5, "random.fuse", 1.0F, 1.0F);
        	}
        	this.checkConnection(par2World, var1[0], var1[1], var1[2]);
        }
        return true;
    }
}