package PuellaMagi.mods;

import java.util.Iterator;
import java.util.List;

import net.minecraft.command.IEntitySelector;
import net.minecraft.enchantment.Enchantment;
import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.ai.EntityAILookIdle;
import net.minecraft.entity.ai.EntityAISwimming;
import net.minecraft.entity.ai.EntityAIWander;
import net.minecraft.entity.ai.EntityAIWatchClosest;
import net.minecraft.entity.item.EntityTNTPrimed;
import net.minecraft.entity.monster.EntityMob;
import net.minecraft.entity.monster.EntitySlime;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.util.MathHelper;
import net.minecraft.world.World;

public class EntitySayaka extends EntityMahoShojo
{
    public EntitySayaka(World par1World)
    {
        super(par1World);
        this.texture = "/PuellaMagi/Mobs/sayaka.png";
        this.MSTexture = "/PuellaMagi/Mobs/sayakaMS.png";
        this.attackStrength = 1;
        this.MSAttackStrength = 12;
        this.MSProtectStrength = 30;
        this.moveSpeed = 0.3F;
        this.soulGemID = mod_QB.sayakaGem.shiftedIndex;
        ItemStack var1 = new ItemStack(mod_QB.sayakaCutlass);

        this.getNavigator().setAvoidsWater(true);
        this.tasks.addTask(1, new EntityAISwimming(this));
        this.tasks.addTask(2, new EntityMadomagiAIAvoidEntity(this, EntityTNTPrimed.class, 16.0F, this.moveSpeed, this.moveSpeed + 0.2F));
        this.tasks.addTask(3, new EntityMadomagiAIStandby(this));
        this.tasks.addTask(4, new EntityMadomagiAIShortRangeAttack(this, this.moveSpeed + 0.1F, 15.0F, var1));
        this.tasks.addTask(5, new EntityMadomagiAIMiddleRangeAttack(this, this.moveSpeed + 0.1F, 5.0F, var1));
        this.tasks.addTask(6, new EntityMadomagiAILongRangeAttack(this, this.moveSpeed - 0.1F, 15.0F, var1));
        this.tasks.addTask(7, new EntityMadomagiAIFollowOwner(this, this.moveSpeed, 1.0F));
        this.tasks.addTask(8, new EntityAIWander(this, this.moveSpeed));
        this.tasks.addTask(9, new EntityAIWatchClosest(this, EntityPlayer.class, 8.0F));
        this.tasks.addTask(9, new EntityAILookIdle(this));
        this.targetTasks.addTask(1, new EntityMadomagiAIOwnerHurtByTarget(this));
        this.targetTasks.addTask(2, new EntityMadomagiAIOwnerHurtTarget(this));
        this.targetTasks.addTask(3, new EntityMadomagiAINearestAttackableTarget(this, EntityMob.class, 16.0F, 2.0F, true));
        this.targetTasks.addTask(4, new EntityMadomagiAINearestAttackableTarget(this, EntitySlime.class, 16.0F, 2.0F, true));
    }

    @Override
    public int getMaxHealth()
    {
        return 20;
    }

    @Override
    public int getHealFrequency()
    {
        return this.isTransformation() ? 15 : super.getHealFrequency();
    }

    @Override
    public int shortRangeAttack(EntityLiving par1EntityLiving, int par2)
    {
        if (par2 <= 0)
        {
        	if (this.checkCutlass(par1EntityLiving))
        	{
        		this.getNavigator().clearPathEntity();
        		return 5;
        	}
        	double var2 = (double)((this.width + par1EntityLiving.width + 0.5D) * (this.width + par1EntityLiving.width + 0.5D));
        	if (this.getDistanceSq(par1EntityLiving.posX, par1EntityLiving.posY, par1EntityLiving.posZ) < var2)
        	{
        		if (this.getHeldItem() != null)
        		{
        			this.swingItem();
        		}
        		this.attackEntityAsMob(par1EntityLiving);
        		this.setSoulGemDamage(1);
        		return 12;
        	}
        }
        return par2;
    }

    @Override
    public int middleRangeAttack(EntityLiving par1EntityLiving, int par2)
    {
        if (par2 <= 0)
        {
        	if (this.checkCutlass(par1EntityLiving))
        	{
        		this.getNavigator().clearPathEntity();
        		return 5;
        	}
        	double var2 = par1EntityLiving.posX - this.posX;
        	double var4 = par1EntityLiving.posY + (double)par1EntityLiving.getEyeHeight() * 0.5D - this.posY;
        	double var6 = par1EntityLiving.posZ - this.posZ;
        	this.motionX = var2 * 0.5F;
        	this.motionY = var4 * 0.5F;
        	this.motionZ = var6 * 0.5F;

        	double var10 = (double)((this.width + par1EntityLiving.width + 0.5D) * (this.width + par1EntityLiving.width + 0.5D));
        	if (this.getDistanceSq(par1EntityLiving.posX, par1EntityLiving.posY, par1EntityLiving.posZ) < var10)
        	{
        		if (this.getHeldItem() != null)
        		{
        			this.swingItem();
        		}
        		this.attackEntityAsMob(par1EntityLiving);
        		this.setSoulGemDamage(1);
        		return 12;
            }
        }
        return par2;
    }

    @Override
    public int longRangeAttack(EntityLiving par1EntityLiving, int par2)
    {
        if (par2 <= 0)
        {
        	if (this.checkCutlass(par1EntityLiving))
        	{
        		this.getNavigator().clearPathEntity();
        		return 5;
        	}
        	else if (this.getNavigator().noPath() && this.onGround)
        	{
        		for (int var0 = 0; var0 < 5; var0++)
        		{
        			float var3 = this.rand.nextFloat() * (float)Math.PI * 2.0F;
        			EntityCutlass var4 = new EntityCutlass(this.worldObj, this.posX + (double)MathHelper.cos(var3), this.posY + this.getEyeHeight(), this.posZ + (double)MathHelper.sin(var3));
        			this.worldObj.spawnEntityInWorld(var4);
        		}
        		this.setSoulGemDamage(1);
        		return 10;
        	}
        }
        return par2;
    }

    public boolean checkCutlass(EntityLiving par1EntityLiving)
    {
        List var1 = this.worldObj.selectEntitiesWithinAABB(EntityCutlass.class, this.boundingBox.expand(3.0D, 3.0D, 3.0D), (IEntitySelector)null);
        Iterator var2 = var1.iterator();
        while (var2.hasNext())
        {
        	Entity var3 = (Entity)var2.next();
        	var3.setDead();
        	EntityCutlass var4 = new EntityCutlass(this.worldObj, this, par1EntityLiving, 2.0F, 6.0F);
        	int var5 = EnchantmentHelper.getEnchantmentLevel(Enchantment.sharpness.effectId, this.getHeldItem()) + EnchantmentHelper.getEnchantmentLevel(Enchantment.smite.effectId, this.getHeldItem()) + EnchantmentHelper.getEnchantmentLevel(Enchantment.baneOfArthropods.effectId, this.getHeldItem()) + 3;
        	int var6 = EnchantmentHelper.getEnchantmentLevel(Enchantment.knockback.effectId, this.getHeldItem());
        	int var7 = EnchantmentHelper.getEnchantmentLevel(Enchantment.fireAspect.effectId, this.getHeldItem());

        	if (var5 > 0)
        	{
        		var4.setDamage(var4.getDamage() + (double)var5 * 0.5D + 0.5D);
        	}
        	if (var6 > 0)
        	{
        		var4.setKnockbackStrength(var6);
        	}
        	if (var7 > 0)
        	{
        		var4.setFire(100);
        	}

        	this.worldObj.playSoundAtEntity(par1EntityLiving, "random.bow", 1.0F, 1.0F / (this.getRNG().nextFloat() * 0.4F + 0.8F));
        	this.worldObj.spawnEntityInWorld(var4);
        	return true;
        }
        return false;
    }
}