package PuellaMagi.mods;

import java.util.Iterator;
import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.ai.EntityAINearestAttackableTarget;
import net.minecraft.entity.boss.IBossDisplayData;
import net.minecraft.entity.item.EntityFireworkRocket;
import net.minecraft.entity.monster.EntityMob;
import net.minecraft.entity.passive.EntityVillager;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemDye;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.DamageSource;
import net.minecraft.util.MathHelper;
import net.minecraft.world.World;
import net.minecraft.world.WorldServer;
import net.minecraft.world.storage.WorldInfo;

public class EntityWalpurgisnacht extends EntityMajo implements IBossDisplayData
{
    private int superArmor;
    private int weatherTimer;

    public EntityWalpurgisnacht(World par1World)
    {
        super(par1World);
        this.texture = "/PuellaMagi/Mobs/walpurgisnacht.png";
        this.setSize(8.0F, 18.0F);
        this.moveSpeed = 0.3F;
        this.superArmor = 0;
        this.weatherTimer = 0;
        this.experienceValue = 500;

        this.tasks.addTask(1, new EntityMajoAIWalpurgisnachtAttack(this, this.moveSpeed * 10.0F));
        this.tasks.addTask(2, new EntityMajoAIWalpurgisnachtPlay(this, this.moveSpeed));
        this.tasks.addTask(3, new EntityMajoAIWalpurgisnachtWander(this, this.moveSpeed));
        this.targetTasks.addTask(0, new EntityMadomagiAINearestAttackableTarget(this, EntityVillager.class, 100.0F, 100.0F, false));
        this.targetTasks.addTask(1, new EntityMadomagiAINearestAttackableTarget(this, EntityPlayer.class, 100.0F, 100.0F, true));
        this.targetTasks.addTask(2, new EntityMadomagiAINearestAttackableTarget(this, EntityMadomagi.class, 100.0F, 100.0F, true));
    }

    @Override
    public int getMaxHealth()
    {
        return 60;
    }

    @Override
    protected boolean canDespawn()
    {
        return this.getAttackTarget() == null && this.getAge() > 11200;
    }

    @Override
    public boolean attackEntityFrom(DamageSource par1DamageSource, int par2)
    {
        int var1 = 1;
        if (par1DamageSource.isFireDamage() || this.superArmor != 0 || (par1DamageSource.getEntity() == null && this.getRNG().nextInt(5) != 0) || par1DamageSource.getEntity() == this)
        {
        	var1 = 0;
        }
        if (par1DamageSource.getEntity() instanceof EntityFireLance || par1DamageSource.getEntity() instanceof EntityPrickle)
        {
        	var1 = 0;
        	return false;
        }
        if (var1 > 0)
        {
        	this.superArmor = 20;
        }
        return super.attackEntityFrom(par1DamageSource, var1);
    }

    @Override
    public void onLivingUpdate()
    {
        super.onLivingUpdate();

        if (this.superArmor > 0)
        {
        	--this.superArmor;
        }

        List var1 = this.worldObj.getCollidingBoundingBoxes(this, this.boundingBox.expand(0.5D, 0.5D, 0.5D));
        Iterator var2 = var1.iterator();
        while (var2.hasNext())
        {
        	AxisAlignedBB var3 = (AxisAlignedBB)var2.next();
        	int var4 = (int)var3.minX;
        	int var5 = (int)var3.minY;
        	int var6 = (int)var3.minZ;
        	int var9 = this.worldObj.getBlockId(var4, var5, var6);
        	if (!this.worldObj.isRemote && var9 > 0 && var9 != Block.bedrock.blockID)
        	{
        		Block var7 = Block.blocksList[this.worldObj.getBlockId(var4, var5, var6)];
        		int var8 = this.worldObj.getBlockMetadata(var4, var5, var6);
        		if (this.worldObj.setBlockWithNotify(var4, var5, var6, 0))
        		{
        			var7.dropBlockAsItemWithChance(this.worldObj, var4, var5, var6, var8, 0.3F, 0);
        		}
        	}
        }

        if (this.weatherTimer-- <= 0 && !this.worldObj.isRemote)
        {
        	this.weatherTimer = 100;
        	WorldServer var4 = MinecraftServer.getServer().worldServers[0];
        	WorldInfo var5 = var4.getWorldInfo();
        	var5.setRainTime(this.weatherTimer);
        	var5.setThunderTime(this.weatherTimer);
        	var5.setRaining(true);
        	var5.setThundering(true);
        	var5.setWorldTime(23200L);
        	var5.incrementTotalWorldTime(23200L);
        }
    }

    @Override
    public boolean getCanSpawnHere()
    {
        if (mod_QB.canWalpurgisSpawn)
        {
        	WorldServer var1 = MinecraftServer.getServer().worldServers[0];
        	WorldInfo var2 = var1.getWorldInfo();
        	int var3 = (int)(var2.getWorldTime() / 24000L) % 8;
        	int var4 = (int)var2.getWorldTime();
        	if (var3 == 0 && var4 > 12000 && var4 < 23200)
        	{
        		int var5 = MathHelper.floor_double(this.posX);
        		int var7 = MathHelper.floor_double(this.boundingBox.minY);
        		int var8 = MathHelper.floor_double(this.posZ);
        		int var9 = this.worldObj.getHeightValue(var5, var8);
        		if (var7 >= var9)
        		{
        			if (super.getCanSpawnHere())
        			{
        				mod_QB.canWalpurgisSpawn = false;
        				for (int var10 = 0; var10 < this.worldObj.playerEntities.size(); ++var10)
        				{
        					EntityPlayer var11 = (EntityPlayer)this.worldObj.playerEntities.get(var10);
        					var11.addChatMessage(" !!!! Walpurgis Night HAS COME !!!!");
        				}
        				return true;
        			}
        		}
        	}
        }
        return false;
    }

    @Override
    public void setDead()
    {
        mod_QB.canWalpurgisSpawn = true;
        super.setDead();
    }

    @Override
    public void onDeath(DamageSource par1DamageSource)
    {
        super.onDeath(par1DamageSource);
        mod_QB.canWalpurgisSpawn = true;
        for (int var12 = 0; var12 < this.worldObj.playerEntities.size(); ++var12)
        {
        	EntityPlayer var13 = (EntityPlayer)this.worldObj.playerEntities.get(var12);
        	if (!this.worldObj.isRemote)
        	{
        		var13.addChatMessage(" Congratulations!!!!!!!!");
        	}
        }

        List var5 = this.worldObj.selectEntitiesWithinAABB(EntityMob.class, this.boundingBox.expand((double)200.0D, (double)200.0D, (double)200.0D), null);
        Iterator var2 = var5.iterator();

        while (var2.hasNext())
        {
        	Entity var3 = (Entity)var2.next();
        	if (var3 != this)
        	{
        		var3.setDead();
        	}
        }
    }

    @Override
    public void onDeathUpdate()
    {
        super.onDeathUpdate();
        if (this.worldObj.isRemote)
        {
        	return;
        }
        for (int var11 = 0; var11 < 4; ++var11)
        {
        	ItemStack var10 = this.getFireWork();
        	double var1 = this.posX + (this.getRNG().nextDouble() + this.getRNG().nextDouble() - 1.0D) * 10.0D;
        	double var3 = this.posZ + (this.getRNG().nextDouble() + this.getRNG().nextDouble() - 1.0D) * 10.0D;
        	int var2 = this.worldObj.getHeightValue((int)var1, (int)var3);
        	EntityFireworkRocket var15 = new EntityFireworkRocket(this.worldObj, var1, (double)var2, var3, var10);
        	this.worldObj.spawnEntityInWorld(var15);
        }
    }

    @Override
    protected void updateAITick()
    {
        this.dataWatcher.updateObject(18, Integer.valueOf(this.getHealth()));
    }

    @Override
    public int getDragonHealth()
    {
       return this.dataWatcher.getWatchableObjectInt(18);
    }

    @Override
    protected void entityInit()
    {
        super.entityInit();
        this.dataWatcher.addObject(18, new Integer(this.getHealth()));
    }

    @Override
    protected void fall(float par1) {}
    @Override
    protected void updateFallState(double par1, boolean par3) {}
    @Override
    public void moveEntityWithHeading(float par1, float par2)
    {
        if (this.isInWater())
        {
        	this.moveFlying(par1, par2, 0.02F);
        	this.moveEntity(this.motionX, this.motionY, this.motionZ);
        	this.motionX *= 0.800000011920929D;
        	this.motionY *= 0.800000011920929D;
        	this.motionZ *= 0.800000011920929D;
        }
        else if (this.handleLavaMovement())
        {
        	this.moveFlying(par1, par2, 0.02F);
        	this.moveEntity(this.motionX, this.motionY, this.motionZ);
        	this.motionX *= 0.5D;
        	this.motionY *= 0.5D;
        	this.motionZ *= 0.5D;
        }
        else
        {
        	float var3 = 0.91F;

        	if (this.onGround)
        	{
        		var3 = 0.54600006F;
        		int var4 = this.worldObj.getBlockId(MathHelper.floor_double(this.posX), MathHelper.floor_double(this.boundingBox.minY) - 1, MathHelper.floor_double(this.posZ));

        		if (var4 > 0)
        		{
        			var3 = Block.blocksList[var4].slipperiness * 0.91F;
        		}
        	}

        	float var8 = 0.16277136F / (var3 * var3 * var3);
        	this.moveFlying(par1, par2, this.onGround ? 0.1F * var8 : 0.02F);
        	var3 = 0.91F;

        	if (this.onGround)
        	{
        		var3 = 0.54600006F;
        		int var5 = this.worldObj.getBlockId(MathHelper.floor_double(this.posX), MathHelper.floor_double(this.boundingBox.minY) - 1, MathHelper.floor_double(this.posZ));

        		if (var5 > 0)
        		{
        			var3 = Block.blocksList[var5].slipperiness * 0.91F;
        		}
        	}

        	this.moveEntity(this.motionX, this.motionY, this.motionZ);
        	this.motionX *= (double)var3;
        	this.motionY *= (double)var3;
        	this.motionZ *= (double)var3;
        }

        this.prevLegYaw = this.legYaw;
        double var10 = this.posX - this.prevPosX;
        double var9 = this.posZ - this.prevPosZ;
        float var7 = MathHelper.sqrt_double(var10 * var10 + var9 * var9) * 4.0F;

        if (var7 > 1.0F)
        {
        	var7 = 1.0F;
        }

        this.legYaw += (var7 - this.legYaw) * 0.4F;
        this.legSwing += this.legYaw;
    }
    @Override
    public boolean isOnLadder()
    {
        return false;
    }

    public float getShaftRotation()
    {
        return -((float)(this.dataWatcher.getWatchableObjectInt(18)) / (float)this.getMaxHealth()) * (float)Math.PI;
    }

    @Override
    protected void dropFewItems(boolean par1, int par2)
    {
        this.dropItem(mod_QB.itemGriefSeed.shiftedIndex, 1);
    }

    @Override
    public int getAttackStrength(Entity par1Entity)
    {
        return 3;
    }

    public void attackFlameLance(EntityLiving par1EntityLiving, int par2)
    {
        for (int var1 = 0; var1 < par2; var1++)
        {
        	EntityFireLance var0 = new EntityFireLance(this.worldObj, this, par1EntityLiving, 1.6F, 6.0F);
        	var0.setFire(100);
        	this.worldObj.playSoundAtEntity(this, "random.bow", 1.0F, 1.0F / (this.getRNG().nextFloat() * 0.4F + 0.8F));
        	this.worldObj.spawnEntityInWorld(var0);
        }
    }

    public void attackPrickle(EntityLiving par1EntityLiving, int par2)
    {
        for (int var1 = 0; var1 < par2; var1++)
        {
            EntityPrickle var0 = new EntityPrickle(this.worldObj, this, par1EntityLiving, 1.6F, 12.0F);
            this.worldObj.playSoundAtEntity(this, "random.bow", 1.0F, 1.0F / (this.getRNG().nextFloat() * 0.4F + 0.8F));
            this.worldObj.spawnEntityInWorld(var0);
        }
    }

    private ItemStack getFireWork()
    {
        ItemStack var0 = new ItemStack(Item.field_92052_bU);
        if (!var0.hasTagCompound())
        {
        	var0.setTagCompound(new NBTTagCompound());
        }
        if (!var0.getTagCompound().hasKey("Fireworks"))
        {
        	var0.setTagCompound(new NBTTagCompound("Fireworks"));
        }
        if (!var0.getTagCompound().hasKey("Explosion"))
        {
        	var0.setTagCompound(new NBTTagCompound("Explosion"));
        }
        NBTTagCompound var15 = var0.getTagCompound();
        NBTTagCompound var18 = var0.getTagCompound().getCompoundTag("Fireworks");
        NBTTagCompound var20 = var0.getTagCompound().getCompoundTag("Explosion");
        NBTTagList var25 = new NBTTagList("Explosions");
        int[] var24 = new int[2];
        var24[0] = ItemDye.dyeColors[this.getRNG().nextInt(16)];
        var24[1] = ItemDye.dyeColors[this.getRNG().nextInt(16)];
        var20.setIntArray("Colors", var24);
        var20.setByte("Type", (byte)this.getRNG().nextInt(3));
        var20.setBoolean("Flicker", this.getRNG().nextInt(2) == 0);
        var20.setBoolean("Trail", this.getRNG().nextInt(2) == 0);
        var25.appendTag(var20);
        var18.setTag("Explosions", var25);
        var18.setByte("Flight", (byte)(this.getRNG().nextInt(2)));
        var15.setTag("Fireworks", var18);
        var0.setTagCompound(var15);
        return var0;
    }
}