package PuellaMagi.mods;

import net.minecraft.enchantment.Enchantment;
import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.EnumAction;
import net.minecraft.item.ItemBow;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.src.ModLoader;
import net.minecraft.world.World;

public abstract class ItemGun extends ItemBow
{
    protected int reloadTime;
    protected int bulletID;
    protected int reloadRatio;
    protected boolean isFullAuto;
    protected float bulletSpeed;
    protected int bulletForce;

    public ItemGun(int par1)
    {
        super(par1);
        this.maxStackSize = 1;
    }

    @Override
    public boolean isFull3D()
    {
        return true;
    }

    @Override
    public void onPlayerStoppedUsing(ItemStack par1ItemStack, World par2World, EntityPlayer par3EntityPlayer, int par4)
    {
        if (!this.isReload(par1ItemStack) && !this.isFullAuto)
        {
        	if (par1ItemStack.getItemDamage() < this.getMaxDamage())
        	{
        		if (this.fireBullet(par1ItemStack, par2World, par3EntityPlayer, this.bulletSpeed, this.itemRand.nextFloat() * -3.0F, 30.0F))
        		{
        			par1ItemStack.damageItem(1, par3EntityPlayer);
        		}
        	}
        	else
        	{
        		//par2World.playSoundAtEntity(par3EntityPlayer, null, 1.0F, 1.0F);
        	}
        }
        this.cancelReload(par1ItemStack, 0x4000);
    }

    @Override
    public ItemStack onFoodEaten(ItemStack par1ItemStack, World par2World, EntityPlayer par3EntityPlayer)
    {
        this.reloadMagazine(par1ItemStack, par2World, par3EntityPlayer);
        return par1ItemStack;
    }

    @Override
    public int getMaxItemUseDuration(ItemStack par1ItemStack)
    {
        int var1 = this.getReload(par1ItemStack);
        if (var1 > -1 && (var1 & 0xf000) < 0x4000)
        {
        	return this.reloadTime();
        }
        else
        {
        	return super.getMaxItemUseDuration(par1ItemStack);
        }
    }

    @Override
    public EnumAction getItemUseAction(ItemStack par1ItemStack)
    {
        return this.isReload(par1ItemStack) ? EnumAction.block : EnumAction.bow;
    }

    @Override
    public ItemStack onItemRightClick(ItemStack par1ItemStack, World par2World, EntityPlayer par3EntityPlayer)
    {
        this.resetBolt(par1ItemStack);
        int var1 = this.getReload(par1ItemStack);
        if (var1 == -1)
        {
        	if (this.canReload(par1ItemStack, par3EntityPlayer))
        	{
        		if (ModLoader.getMinecraftInstance().gameSettings.keyBindAttack.pressed && !this.isWeaponFullAuto(par1ItemStack))
        		{
        			this.setReload(par1ItemStack, 0);
        			var1 = 0;
        		}

        		if (this.isEmpty(par1ItemStack))
        		{
        			this.setReload(par1ItemStack, 0);
        			var1 = 0;
        		}
        	}
        }
        if (var1 == 0)
        {
        	this.releaseMagazine(par1ItemStack, par2World, par3EntityPlayer);
        }
        par3EntityPlayer.setItemInUse(par1ItemStack, this.getMaxItemUseDuration(par1ItemStack));
        return par1ItemStack;
    }

    @Override
    public void onUpdate(ItemStack par1ItemStack, World par2World, Entity par3Entity, int par4, boolean par5)
    {
        if (!this.isReload(par1ItemStack) && this.isWeaponFullAuto(par1ItemStack))
        {
        	if (par3Entity != null && par3Entity instanceof EntityPlayer)
        	{
        		EntityPlayer var1EntityPlayer = (EntityPlayer)par3Entity;
        		if (par1ItemStack != null && var1EntityPlayer.getItemInUse() == par1ItemStack && this.cycleBolt(par1ItemStack))
        		{
        			if (par1ItemStack.getItemDamage() < this.getMaxDamage() && ModLoader.getMinecraftInstance().gameSettings.keyBindUseItem.pressed)
        			{
        				if (this.fireBullet(par1ItemStack, par2World, var1EntityPlayer, this.bulletSpeed, this.itemRand.nextFloat() * -0.3F, 35.8F))
        				{
        					par1ItemStack.damageItem(1, var1EntityPlayer);
        				}
        				//par2World.playSoundAtEntity(var1EntityPlayer, "random.click", 1.0F, 1.0F / (itemRand.nextFloat() * 0.4F + 1.2F));
        			}
        			else if(this.canReload(par1ItemStack, var1EntityPlayer))
        			{
        				var1EntityPlayer.stopUsingItem();
        			}
        		}
        	}
        }
        super.onUpdate(par1ItemStack, par2World, par3Entity, par4, par5);
    }

    public int reloadTime()
    {
        return this.reloadTime;
    }

    public boolean canReload(ItemStack par1ItemStack, EntityPlayer par2EntityPlayer)
    {
        if (par2EntityPlayer.capabilities.isCreativeMode || par2EntityPlayer.inventory.hasItem(this.getBulletID(par1ItemStack)))
        {
        	return true;
        }
        return false;
    }

    public boolean isReload(ItemStack par1ItemStack)
    {
        return this.getReload(par1ItemStack) > -1;
    }

    public void setReload(ItemStack par1ItemStack, int par2)
    {
        if (!par1ItemStack.hasTagCompound())
        {
        	par1ItemStack.setTagCompound(new NBTTagCompound());
        }
        NBTTagCompound var1 = par1ItemStack.getTagCompound();
        var1.setInteger("Reload", par2);
    }

    public int getReload(ItemStack par1ItemStack)
    {
        if (!par1ItemStack.hasTagCompound())
        {
        	this.setReload(par1ItemStack, -1);
        	return -1;
        }
        NBTTagCompound var1 = par1ItemStack.getTagCompound();
        return var1.getInteger("Reload");
    }

    public void cancelReload(ItemStack par1ItemStack, int par2)
    {
        if (this.getReload(par1ItemStack) >= par2)
        {
        	this.setReload(par1ItemStack, -1);
        }
    }

    public boolean isEmpty(ItemStack par1ItemStack)
    {
        return par1ItemStack.getItemDamage() >= this.getMaxDamage();
    }

    public void releaseMagazine(ItemStack par1ItemStack, World par2World, EntityPlayer par3EntityPlayer)
    {
        this.setReload(par1ItemStack, (0x1000 | par1ItemStack.getItemDamage()));
        par1ItemStack.setItemDamage(this.getMaxDamage());
    }

    public void reloadMagazine(ItemStack par1ItemStack, World par2World, EntityPlayer par3EntityPlayer)
    {
        if (!par2World.isRemote)
        {
        	if (par3EntityPlayer == null || par3EntityPlayer.capabilities.isCreativeMode || EnchantmentHelper.getEnchantmentLevel(Enchantment.infinity.effectId, par1ItemStack) > 0)
        	{
        		par1ItemStack.setItemDamage(0);
        	}
        	else
        	{
        		int var1 = this.getReload(par1ItemStack);
        		var1 = (var1 > 0) ? var1 & 0x0fff : 0;
        		int var2 = 0;
        		while (true)
        		{
        			if (var1 == 0 || !par3EntityPlayer.inventory.hasItem(this.getBulletID(par1ItemStack)))
        			{
        				break;
        			}
        			var1--;
        			if (++var2 >= this.reloadRatio)
        			{
        				var2 = 0;
        				par3EntityPlayer.inventory.consumeInventoryItem(this.getBulletID(par1ItemStack));
        			}
        		}
        		par1ItemStack.setItemDamage(var1);
        	}
        }
        this.setReload(par1ItemStack, 0x4000);
    }

    public int getBulletID(ItemStack par1ItemStack)
    {
        return this.bulletID;
    }

    public boolean fireBullet(ItemStack par1ItemStack, World par2World, EntityPlayer par3EntityPlayer, float par4, float par5, float par6)
    {
        if (!par2World.isRemote)
        {
        	EntityMadomagiBullet var1 = new EntityMadomagiBullet(par2World, par3EntityPlayer, par4);

        	// Power
        	int var2 = EnchantmentHelper.getEnchantmentLevel(Enchantment.power.effectId, par1ItemStack) + this.bulletForce;
        	if (var2 > 0)
        	{
        		var1.setDamagePower(var1.getDamagePower() + (double)var2 * 0.5D + 0.5D);
        	}

        	// Punch
        	int var3 = EnchantmentHelper.getEnchantmentLevel(Enchantment.punch.effectId, par1ItemStack);
        	if (var3 > 0)
        	{
        		var1.setKnockbackStrength(var3);
        	}

        	// Fire
        	if (EnchantmentHelper.getEnchantmentLevel(Enchantment.flame.effectId, par1ItemStack) > 0)
        	{
        		var1.setFire(100);
        	}

        	par2World.playSoundAtEntity(par3EntityPlayer, "random.bow", 1.0F, 1.0F / (itemRand.nextFloat() * 0.4F + 1.2F) + par4 * 0.125F);
        	par2World.spawnEntityInWorld(var1);
        }
        par3EntityPlayer.rotationPitch += par5;
        par3EntityPlayer.rotationYaw = par3EntityPlayer.rotationYaw + (this.itemRand.nextFloat() - 0.5F) * par5;
        return (EnchantmentHelper.getEnchantmentLevel(Enchantment.infinity.effectId, par1ItemStack) <= 0) && !par3EntityPlayer.capabilities.isCreativeMode;
    }

    public boolean isWeaponFullAuto(ItemStack par1ItemStack)
    {
        return this.isFullAuto;
    }

    public void resetBolt(ItemStack par1ItemStack)
    {
        if (!par1ItemStack.hasTagCompound())
        {
        	par1ItemStack.setTagCompound(new NBTTagCompound());
        }
        par1ItemStack.getTagCompound().setBoolean("Bolt", false);
    }

    public boolean cycleBolt(ItemStack par1ItemStack)
    {
        if (!par1ItemStack.hasTagCompound())
        {
        	par1ItemStack.setTagCompound(new NBTTagCompound());
        }
        NBTTagCompound var1 = par1ItemStack.getTagCompound();
        boolean var2 = !(var1.getBoolean("Bolt"));
        var1.setBoolean("Bolt", var2);
        return var2;
    }

    public boolean getBolt(ItemStack par1ItemStack)
    {
        if (!par1ItemStack.hasTagCompound())
        {
        	return false;
        }
        NBTTagCompound var1 = par1ItemStack.getTagCompound();
        return var1.getBoolean("Bolt");
    }

    @Override
    public String getTextureFile()
    {
        return "/PuellaMagi/Items/madomagiItem.png";
    }
}